%% Computing time derivatives of state variables
%
% Copyright:
% Jason Bazil, Kalyan Vinnakota, and Dan Beard. April 2015
% Beard Laboratory
% Molecular and Integrative Physiology
% University of Michigan

function varargout = DAEs(t,x,p)
% initialize LHS
f = zeros(size(x,1),size(x,2));

%% Compute binding polynomial array
p.BP = ones(size(x,1),size(x,2));
p.BP(p.i.Lmtx,:) = BP(p.if.ligands,'mtx',x,p);
p.BP(p.i.Lcyt,:) = BP(p.if.ligands,'cyt',x,p);

%% Compute Algebriac Constraint Residuals

% Compute other Matrix Conserved Species
f(p.i.ATP_x,:) = x(p.i.ATP_x,:) - (p.Atot - x(p.i.ADP_x,:));
f(p.i.NADH_x,:) = x(p.i.NADH_x,:) - (p.Ntot - x(p.i.NAD_x,:));
f(p.i.UQH2_x,:) = x(p.i.UQH2_x,:) - (p.Qtot - x(p.i.UQ_x,:));
f(p.i.c2_i,:) = x(p.i.c2_i,:) - (p.Ctot - x(p.i.c3_i,:));
f(p.i.PCr_c,:) = x(p.i.PCr_c,:) - (p.Crtot - x(p.i.Cr_c,:));

%% Compute TCA Reaction Rates (nmol/min/mg)

J.MitoDH = MitoDH(x,p);   % NAD+ = NADH + H+ w/ saturable ADP*Pi/ATP regulation

%% Compute ETS Fluxes (nmol/min/mg)

[J.CI.JNADH,J.CI.JQ,J.CI.JSO,J.CI.JH2O2] = ETC1(x,p);       % NADH + UQ + 5Hmtx+ = NAD + UQH2 + 4Hims+

if p.IRsimulation
    J.CII = ETC2m1(x,p);
else
    J.CII = ETC2m0(x,p,J.MitoDH);
end

[J.CIII.Jc3,J.CIII.JQH2,J.CIII.JQ,J.CIII.JSO,J.CIII.JHp,J.CIII.JHn,J.CIII.JdPsi] = ETC3(x,p);   % UQH2 + 2Cytc(ox) = UQ + 2Cytc(red) + 2Hims+

J.CIV = ETC4(x,p);     % 2Cytc(red) + 1/2O2 + 4Hmtx+ = 2Cytc(ox) + H20 + 2Hims+

J.mHleak = HLEAK(x,p);

J.FCCP = p.FCCP*(x(p.i.dPsi,:) - 0); % FCCP flux

%% Compute ATP/ADP cycle Fluxes

J.F1FO = F1FOATPASE(x,p);     % ADP + Pi + 3Hims+ = ATP + 2Hims+

J.ANT = ANT(x,p); % Metelkin et al. 2006  

J.PIC = PIH(x,p); 

%% Compute K cycle Fluxes

J.mKHE = KHE(x,p);

%% ROS Scavenging (currently, place-holders)
% rates derived from values in Murphy's 2012 ARS paper and his 2009 Biochem J paper
J.SODx = 2e4*x(p.i.SO_x,:); % assuming 10uM MnSOD and k of 2e9 M^-1*s^1 (M/s)
J.SODc = 2e4*x(p.i.SO_c,:); % assuming similiar rate for matrix (M/s)
J.H2O2scavenge = 1320*x(p.i.H2O2_x,:); % assuming 60 uM Prx3 and k of 2e7 M^-1*s^1 and 2 uM Gpx1 and k of 6e7 M^-1*s^1 (M/s)
J.H2O2perm = 333*(x(p.i.H2O2_x,:) - x(p.i.H2O2_c,:)); % based information in Bienert et al. 2006 BBA review and 6 nm membrane thickness (2 um/s / 6 nm) wrt to matrix volume

%% Compute Time Derivatives for Matrix non-cation State Variables

% dPsi
f(p.i.dPsi,:) = (+ 4*J.CI.JNADH + 2*J.CIII.JQH2 + 4*J.CIV - p.F1FO.nH*J.F1FO - 1*J.ANT - 1*J.mHleak - J.FCCP)/p.Cmito;

% Matrix species
% f(p.i.ATP_x,:) = -(J.ANT - J.F1FO)/p.Vol.H2O_mtx;
f(p.i.ADP_x,:) = (+ 1*J.ANT - 1*J.F1FO)/p.Vol.H2O_mtx;
f(p.i.Pi_x,:) = (+ 1*J.PIC - 1*J.F1FO)/p.Vol.H2O_mtx;
% f(p.i.NADH_x,:) = (J.MitoDH - J.CI.JNADH)/p.Vol.H2O_mtx;
f(p.i.NAD_x,:) = (- 1*J.MitoDH + 1*J.CI.JNADH)/p.Vol.H2O_mtx;
% f(p.i.UQH2_x,:) = (J.CI.JQ + J.CII - J.CIII.JQ)/p.Vol.H2O_mtx;
f(p.i.UQ_x,:) = (- 1*J.CI.JNADH - 1*J.CII + 1*J.CIII.JQH2)/p.Vol.H2O_mtx;
f(p.i.SO_x,:) = (+ 1*J.CI.JSO + 1/2*J.CIII.JSO)/p.Vol.H2O_mtx - 2*J.SODx;
f(p.i.H2O2_x,:) = (+ 1*J.CI.JH2O2)/p.Vol.H2O_mtx + J.SODx - 1*J.H2O2scavenge - 1*J.H2O2perm;

%% Compute Matrix Cation Derivatives Based on Rapid Equilibrium Dynamics

f = mCation_dXdt(x,f,p,J);

%% Compute Time Derivatives for IMS non-cation State Variables

% f(p.i.c2_i,:) = (J.CIII.Jc3 - 2*J.CIV)/p.Vol.H2O_ims;
f(p.i.c3_i,:) = (- 2*J.CIII.JQH2 + 2*J.CIV)/p.Vol.H2O_ims;

%% External ATPase

if p.IRsimulation
    J.ATPase = ATPasem1(x,p); % Wu et al. 2009 ATPase
    J.CK = CKm0(x,p);
    J.AK = AKm0(x,p);
else
    J.ATPase = ATPasem0(x,p); % ATPase for Vinnakota et al. 2015
    J.CK=zeros(1,size(x,2));
    J.AK=zeros(1,size(x,2));
end

%% Compute Derivative for cytosolic/buffer space

% Buffer/Cytosolic species
f(p.i.Pi_c,:) = (- 1*J.PIC)/p.Vol.H2O_cyt + 1*J.ATPase;
f(p.i.ATP_c,:) = ( + 1*J.ANT)/p.Vol.H2O_cyt - 1*J.ATPase + 1*J.AK + 1*J.CK;
f(p.i.ADP_c,:) = ( - 1*J.ANT)/p.Vol.H2O_cyt + 1*J.ATPase - 2*1*J.AK - 1*J.CK;
f(p.i.AMP_c,:) = 1*J.AK;
f(p.i.Cr_c,:) = 1*J.CK;
f(p.i.SO_c,:) = (+ 1/2*J.CIII.JSO)/p.Vol.H2O_cyt  - 2*J.SODc;
f(p.i.H2O2_c,:) = J.SODc + 1*J.H2O2perm*p.Vol.H2O_mtx/p.Vol.H2O_cyt; % H2O2perm in units of M/s wrt matrix H2O, H2O_cyt in units of l buffer/lmito, H2O_mtx converts l buffer/lmito to lbuffer/lH2Omtx

%% extra output arguments
varargout={f,J};
end

%% Binding Polynomial Calculation
% Only 1st order binding species are considered assuming that higher
% order binding species concentractions are neglegible.
% inputs:
% l - ligand of interest
% c - compartment, 1) mtx (mito matrix), 2) ims (imm space), 
%                  3) cyt (cytosol), etc.
% x - state variable vector
% p - parameter structure
% output:
% f - free ligand conc. based on free protons and metal ion conc.

function [f, flag, new_data] = BP(l,c,x,p)

if strcmp(c,'mtx')==1
    % indices for matrix cations
    i=[p.i.H_x,p.i.K_x,p.i.Na_x,p.i.Mg_x,p.i.Ca_x];
elseif strcmp(c,'ims')==1
    % indices for IMS cations
%     i=[p.i.H_i,p.i.K_i,p.i.Na_i,p.i.Mg_i,p.i.Ca_i]; 
%     v=p.Vims;
    % indices for IMS cations replaced w/ Cytosolic cations
    i=[p.i.H_c,p.i.K_c,p.i.Na_c,p.i.Mg_c,p.i.Ca_c]; 
elseif strcmp(c,'cyt')==1
    % indices for matrix cations
    i=[p.i.H_c,p.i.K_c,p.i.Na_c,p.i.Mg_c,p.i.Ca_c]; 
else %add further compartments as necessary
    error('currently, only matrix, ims or cytosol arguments are accepted')
end

% define appropriate cations
H = x(i(1),:);
K = x(i(2),:);
Na = x(i(3),:);
Mg = x(i(4),:);
Ca = x(i(5),:);

% this is if the code is not vectorized
%f=(1 + H./p.KD(l,p.if.H) + K./p.KD(l,p.if.K) + Na./p.KD(l,p.if.Na) ...
%    + Mg./p.KD(l,p.if.Mg) + Ca./p.KD(l,p.if.Ca));

% this is for vectorized code
f = (1 + (1./p.KD(l,p.if.H))*H + (1./p.KD(l,p.if.K))*K ...
    + (1./p.KD(l,p.if.Na)*Na) + (1./p.KD(l,p.if.Mg))*Mg ...
    + (1./p.KD(l,p.if.Ca))*Ca);

flag=0;

new_data=0;
end

%% Compute Ion Derivatives Using Rapid Equilibrium Approximation Method
function f = mCation_dXdt(x,f,p,J)
% Compute Time Derivatives for Cations
H_x = x(p.i.H_x,:);     % M
K_x = x(p.i.K_x,:);     % M
Na_x = x(p.i.Na_x,:);   % M
Mg_x = x(p.i.Mg_x,:);   % M
Ca_x = x(p.i.Ca_x,:);   % M

% Define Ligand Vector
Li = x(p.i.Lmtx,:); % M

% Define Ligand Derivatives
dLidt = f(p.i.Lmtx,:); % M/s
% set dLidt for algebraic constraints equal to negative of counterpart
dLidt(1,:) = -f(p.i.ADP_x,:);
dLidt(7,:) = -f(p.i.NAD_x,:);

% Partial Derivatives of binding polynomials:
Pm_L = p.BP(p.i.Lmtx,:);
Pm_L2 = Pm_L.^2;
Pm_HBH = (1 + 1./p.KD(p.if.ligands,p.if.K)*K_x + 1./p.KD(p.if.ligands,p.if.Na)*Na_x ...
    + 1./p.KD(p.if.ligands,p.if.Mg)*Mg_x + 1./p.KD(p.if.ligands,p.if.Ca)*Ca_x);
Pm_KBK = (1 + 1./p.KD(p.if.ligands,p.if.H)*H_x + 1./p.KD(p.if.ligands,p.if.Na)*Na_x ...
    + 1./p.KD(p.if.ligands,p.if.Mg)*Mg_x + 1./p.KD(p.if.ligands,p.if.Ca)*Ca_x);
Pm_MBM = (1 + 1./p.KD(p.if.ligands,p.if.H)*H_x + 1./p.KD(p.if.ligands,p.if.K)*K_x ...
    + 1./p.KD(p.if.ligands,p.if.Na)*Na_x + 1./p.KD(p.if.ligands,p.if.Ca)*Ca_x);

pHBpH = + sum(Pm_HBH.*Li./Pm_L2./(p.KD(p.if.ligands,p.if.H)*ones(1,size(Li,2))),1);
pHBpK = - H_x.*sum(Li./Pm_L2 ...
    ./(p.KD(p.if.ligands,p.if.H).*p.KD(p.if.ligands,p.if.K)*ones(1,size(Li,2))),1);
pHBpMg = - H_x.*sum(Li./Pm_L2 ...
    ./(p.KD(p.if.ligands,p.if.H).*p.KD(p.if.ligands,p.if.Mg)*ones(1,size(Li,2))),1);

pKBpH = - K_x.*sum(Li./Pm_L2 ...
    ./(p.KD(p.if.ligands,p.if.K).*p.KD(p.if.ligands,p.if.H)*ones(1,size(Li,2))),1);
pKBpK = + sum(Pm_KBK.*Li./Pm_L2./(p.KD(p.if.ligands,p.if.K)*ones(1,size(Li,2))),1);
pKBpMg = - K_x.*sum(Li./Pm_L2 ...
    ./(p.KD(p.if.ligands,p.if.K).*p.KD(p.if.ligands,p.if.Mg)*ones(1,size(Li,2))),1);

pMgBpH = - Mg_x.*sum(Li./Pm_L2 ...
    ./(p.KD(p.if.ligands,p.if.Mg).*p.KD(p.if.ligands,p.if.H)*ones(1,size(Li,2))),1);
pMgBpK = - Mg_x.*sum(Li./Pm_L2 ...
    ./(p.KD(p.if.ligands,p.if.Mg).*p.KD(p.if.ligands,p.if.K)*ones(1,size(Li,2))),1);
pMgBpMg = + sum(Pm_MBM.*Li./Pm_L2./(p.KD(p.if.ligands,p.if.Mg)*ones(1,size(Li,2))),1);

% PHI's:
PhiH = - H_x.*sum(dLidt./Pm_L ...
    ./(p.KD(p.if.ligands,p.if.H)*ones(1,size(Li,2)))) ...
    + (J.MitoDH - 5*J.CI.JNADH - 2*J.CIII.JQH2 - 4*J.CIV ...
    + (p.F1FO.nH-1)*J.F1FO + 2*J.PIC + J.mHleak ...
    + 1*J.mKHE + J.FCCP)/p.Vol.H2O_mtx;% - 2*J.mCa2H;
PhiK = - K_x.*sum(dLidt./Pm_L ...
    ./(p.KD(p.if.ligands,p.if.K)*ones(1,size(Li,2)))) ...
    + ( - J.mKHE)/p.Vol.H2O_mtx;
PhiMg = - Mg_x.*sum(dLidt./Pm_L ...
    ./(p.KD(p.if.ligands,p.if.Mg)*ones(1,size(Li,2))));

% Endogenous buffering for [H+], [K+], [Na+], [Mg2+] and [Ca2+]
BH = p.Buff.Htot;       % M
K_BH = p.Buff.KD_H;     % M
BMg = p.Buff.Mgtot;     % M
K_BMg = p.Buff.KD_Mg;   % M

% Buffering Terms:
DH = 1 + pHBpH + BH./K_BH./(1+H_x./K_BH).^2;
DK = 1 + pKBpK;
DMg = 1 + pMgBpMg + BMg./K_BMg./(1+Mg_x./K_BMg).^2;

% DENOMINATOR
D = DH.*pKBpMg.*pMgBpK + DK.*pHBpMg.*pMgBpH + DMg.*pHBpK.*pKBpH - ...
    DMg.*DK.*DH - pHBpK.*pKBpMg.*pMgBpH - pHBpMg.*pMgBpK.*pKBpH;
% DERIVATIVES for H,Mg,K (M/s)
f(p.i.H_x,:) = ( (pKBpMg.*pMgBpK - DMg.*DK).*PhiH + ...
            (DK.*pHBpMg - pHBpK.*pKBpMg).*PhiMg + ...
            (DMg.*pHBpK - pHBpMg.*pMgBpK).*PhiK ) ./ D;
f(p.i.K_x,:) = ( (DMg.*pKBpH - pKBpMg.*pMgBpH).*PhiH + ...
            (DH.*pKBpMg - pKBpH.*pHBpMg).*PhiMg + ...
            (pMgBpH.*pHBpMg - DH.*DMg).*PhiK ) ./ D;
f(p.i.Mg_x,:) = ( (DK.*pMgBpH - pKBpH.*pMgBpK).*PhiH + ...
            (pKBpH.*pHBpK - DH.*DK).*PhiMg + ...
            (DH.*pMgBpK - pHBpK.*pMgBpH).*PhiK ) ./ D;
end
%% Substrate oxidation
function J = MitoDH(x,p)

% Define SVs into user friendly notation
NAD = x(p.i.NAD_x,:);   % M
NADH = x(p.i.NADH_x,:); % M
Pi = x(p.i.Pi_x,:);     % M
ATP = x(p.i.ATP_x,:);     % M
ADP = x(p.i.ADP_x,:);     % M

r = (ADP.*Pi./ATP); % feedback ratio

J = p.MitoDH.x*r.^p.MitoDH.n./(r.^p.MitoDH.n + p.MitoDH.K.^p.MitoDH.n).*(p.MitoDH.r*min(NAD,2.58e-3) - NADH); % NAD saturation added

end

%% CI model that includes ROS pathways (5-State Model)
%
% 5 State Model
% E0: 0e-
% E1: 1e-
% E2: 2e-
% E3: 3e-
% E4: 4e-
function varargout = ETC1(x,p)
% define state variables
Hp = x(p.i.H_c,:);
Hn = x(p.i.H_x,:);
NAD = x(p.i.NAD_x,:);
NADH = x(p.i.NADH_x,:);
Q = x(p.i.UQ_x,:);
QH2 = x(p.i.UQH2_x,:);
dPsi = x(p.i.dPsi,:);
O2 = x(p.i.O2_x,:);
SO = x(p.i.SO_x,:);
H2O2 = x(p.i.H2O2_x,:);

% Update midpoint potentials from thermodynamic data
p.CI.Em0.Q_QH2 = (p.dGf(p.if.UQH2) - p.dGf(p.if.UQ))/(-2*p.F); % this is 18.67 mV less than value used for CI parameterization
p.CI.Em0.NADH = (p.dGf(p.if.NADH) - p.dGf(p.if.NAD))/(-2*p.F); % this is 2.3 mV higher than value used for CI parameterization
p.CI.Em0.SO = (p.dGf(p.if.SO) - p.dGf(p.if.O2))/(-1*p.F); % this is 8.3 mV higher than value used for CI parameterization
p.CI.Em0.H2O2 = (p.dGf(p.if.H2O2) - p.dGf(p.if.O2))/(-2*p.F); % this is 8.4 mV less than value used for CI parameterization

% Update midpoint potentials for 1e- quinone couples (pH 0)
p.CI.Em0.Q_SQ = p.CI.Em0.Q_QH2 + p.R*p.T./p.F./2*log(p.CI.Kstability*1e-14); % mV
p.CI.Em0.SQ_QH2 = 2*p.CI.Em0.Q_QH2 - p.CI.Em0.Q_SQ; % mV

% Binding Polynomials for Protonated States
PH1 = (1./(1+Hn./p.CI.KiH1));
PH2 = 1;
PH3 = (Hn./p.CI.KiH3./(1+Hn./p.CI.KiH3));

% Binding Polynomials for enzyme, substrates, products and regulators
% NADH binding constants
KdNADHo=p.CI.K_NADH_o;
KdNADo=p.CI.K_NAD_o;
KdNADHr=p.CI.K_NADH_r;
KdNADr=p.CI.K_NAD_r;
KdNADHrad=p.CI.K_NADH_rad;
KdNADrad=p.CI.K_NAD_rad;
% Quinone binding constants
KdQH2o = p.CI.K_QH2;
KdQo = p.CI.K_Q;
KdQH2r = p.CI.K_QH2;
KdQr = p.CI.K_Q;
% Binding polynomials
PNo = 1 + NADH./KdNADHo + NAD./KdNADo; % oxidized binding constants
PNr = 1 + NADH./KdNADHr + NAD./KdNADr; % reduced binding constants
PNrad = 1 + NADH./KdNADHrad + NAD./KdNADrad; % reduced binding constants
PQr = 1 + QH2./KdQH2r + Q./KdQr; % reduced binding constants
PQo = 1 + QH2./KdQH2o + Q./KdQo; % oxidized binding constants

muH = p.F*dPsi+p.R*p.T*log(Hp./Hn); % proton chemical potential (kJ/mol)

% Compute pH Corrected Midpoint potentials
% NADH potential
Em.NADH = p.CI.Em0.NADH + log(10).*p.R.*p.T./p.F./2.*log10(Hn);
% FMN potentials
Em.FMNred = p.CI.Em0.FMN - p.R*p.T./2./p.F*log(Hn./10^-p.CI.FMNred.pK./(1+Hn./10^-p.CI.FMNred.pK)./Hn.^2); % FMNred./FMN, pK1 of fully reduced FMN very high (>10)0)
Em.FMNrad = p.CI.Em0.FMN2 - p.R*p.T./p.F*log((Hn./10^-(p.CI.FMNred.pK)./(1+Hn./10^-p.CI.FMNred.pK))./((Hn./10^-p.CI.FMNrad.pK)./(1+Hn./10^-p.CI.FMNrad.pK))./Hn);  % FMNred./FMNrad
Em.FMNox = p.CI.Em0.FMN1 - p.R*p.T./p.F*log((Hn./10^-p.CI.FMNrad.pK)./(1+Hn./10^-p.CI.FMNrad.pK)./Hn); %FMN./FMNrad
% N2 potential
Em.N2 = p.CI.Em0.N2 - p.R*p.T./p.F*log((Hn+10^-p.CI.N2.pKox)./(Hn+10^-p.CI.N2.pKred));
% superoxide and H2O2 potentials
Em.SO = p.CI.Em0.SO; % Em0 stays the same for pH > 6, pKa of SO is ~4.8
Em.H2O2 = p.CI.Em0.H2O2 + log(10).*p.R.*p.T./p.F.*log10(Hn); % 1st pKa is ~11
% Quinone potentials
Em.Q_QH2 = p.CI.Em0.Q_QH2 + 2*log(10)*p.R*p.T./p.F./2*log10(Hn); % assuming linked to N-side
Em.Q_SQ = p.CI.Em0.Q_SQ; % pH independent
Em.SQ_QH2 = p.CI.Em0.SQ_QH2 + 2*log(10)*p.R*p.T./p.F*log10(Hn); % assuming linked to N-side

% State Transition Thermodynamics
% NADH-QH2 reductase related
Keq.NADHFMNred = exp((2.*p.F.*(Em.FMNred - Em.NADH))./p.R./p.T); % [Fr][NAD]./[F][NADH][H]
Keq.FMNradN2 = exp((p.F.*(Em.N2 - Em.FMNrad))./p.R./p.T); % [Frad][N2r]./[Fr]./[N2]
Keq.FMNoxN2 = exp((p.F.*(Em.N2 - Em.FMNox))./p.R./p.T); % [F][N2r]./[Frad]./[N2]
Keq.N2SQ = exp((p.F.*(Em.Q_SQ - Em.N2))./p.R./p.T); % [N2][SQ]./[N2r][Q]
Keq.N2QH2 = exp((p.F.*(Em.SQ_QH2 - Em.N2))./p.R./p.T); % [N2][QH2]./[N2r][SQ]

% superoxide and hydrogen peroxide
Keq.FMNredH2O2 = exp((2.*p.F.*(Em.H2O2 - Em.FMNred))./p.R./p.T); % [F][H2O2]./[Fr][O2]
Keq.FMNradO2 = exp((p.F.*(Em.SO - Em.FMNrad))./p.R./p.T); % [Frad][O2.-]./[Fr][O2]
Keq.FMNoxO2 = exp((p.F.*(Em.SO - Em.FMNox))./p.R./p.T); % [F][O2.-]./[Frad][O2]
Keq.N2O2 = exp((p.F.*(Em.SO - Em.N2))./p.R./p.T); % [N2][O2.-]./[N2r][O2]
Keq.SQO2 = exp((p.F.*(Em.SO - Em.Q_SQ))./p.R./p.T); % [Q][O2.-]./[SQ][O2]

% Substates via Rapid Equilibrium

% substate equilibrium constants
K1 = Keq.FMNradN2; %[Frad][N2r]./[Fr][N2]
K2 = Keq.FMNoxN2; % [F][N2r]./[Frad][N2]
KSQ = Keq.N2SQ.*KdQr; % [N2-SQ]./[N2r-Q], KdQr = [N2r][Q]./[N2r-Q]

KQ = (Q./KdQr./PQr).*KSQ; % [N2-SQ]./[N2r]

% 0e-
PS0 = 1; % total
s0.F_N2 = 1./PS0;

% 1e-
PS1 = (1 + K2.*(1 + KQ)); % total sum(cell2mat(struct2cell(s1)))
s1.Frad_N2 = 1./PS1;
s1.F_N2r = K2.*s1.Frad_N2;
s1.F_N2_SQ = KQ.*s1.F_N2r;

% 2e-
PS2 = (1 + K1.*(1 + KQ.*(1 + K2))); % total sum(cell2mat(struct2cell(s2)))
s2.Fr_N2 = 1./PS2;
s2.Frad_N2r = K1.*s2.Fr_N2;
s2.Frad_N2_SQ = KQ.*s2.Frad_N2r;
s2.F_N2r_SQ = K2.*s2.Frad_N2_SQ;

% 3e-
PS3 = (1 + KQ.*(1 + K1)); % total sum(cell2mat(struct2cell(s3)))
s3.Fr_N2r = 1./PS3;
s3.Fr_N2_SQ = KQ.*s3.Fr_N2r;
s3.Frad_N2r_SQ = K1.*s3.Fr_N2_SQ;

% 4e-
PS4 = 1; % total
s4.Fr_N2r_SQ = 1./PS4;


% State Transition Rates

% reverse NADH-Q reductase rates
krN0_02 = p.CI.kfNADH_02./Keq.NADHFMNred*KdNADr./KdNADHo.*PH1.*PH2;
krN0_24 = p.CI.kfNADH_24./Keq.NADHFMNred*KdNADr./KdNADHo.*PH1.*PH2;
krN0_13 = p.CI.kfNADH_13./Keq.NADHFMNred*KdNADr./KdNADHo.*PH1.*PH2;
krQ0_20 = p.CI.kfQ_20./Keq.N2QH2*KdQH2o.*(PNr.*PQo./PNo./PQr).*PH3;
krQ0_42 = p.CI.kfQ_42./Keq.N2QH2*KdQH2o.*(PNr.*PQo./PNo./PQr).*PH3;
krQ0_31 = p.CI.kfQ_31./Keq.N2QH2*KdQH2o.*(PNr.*PQo./PNo./PQr).*PH3;

% reverse ROS rates
% O2 and SQ
krSQSO_10 = p.CI.kfSQSO_10./Keq.SQO2;
krSQSO_21 = p.CI.kfSQSO_21./Keq.SQO2;
krSQSO_21b = p.CI.kfSQSO_21b./Keq.SQO2;
krSQSO_32 = p.CI.kfSQSO_32./Keq.SQO2;
krSQSO_32b = p.CI.kfSQSO_32b./Keq.SQO2;
krSQSO_43 = p.CI.kfSQSO_43./Keq.SQO2;
% O2 and N2
krN2rSO_10 = p.CI.kfN2rSO_10./Keq.N2O2;
krN2rSO_21 = p.CI.kfN2rSO_21./Keq.N2O2;
krN2rSO_21b = p.CI.kfN2rSO_21b./Keq.N2O2;
krN2rSO_32 = p.CI.kfN2rSO_32./Keq.N2O2;
krN2rSO_32b = p.CI.kfN2rSO_32b./Keq.N2O2;
krN2rSO_43 = p.CI.kfN2rSO_43./Keq.N2O2;
% O2 and FMNred
krFrSO_21 = p.CI.kfFrSO_21./Keq.FMNradO2;
krFradSO_21b = p.CI.kfFradSO_21b./Keq.FMNradO2;
krFrSO_32 = p.CI.kfFrSO_32./Keq.FMNradO2;
krFrSO_32b = p.CI.kfFrSO_32b./Keq.FMNradO2;
krFrSO_43 = p.CI.kfFrSO_43./Keq.FMNradO2;
% O2 and FMNrad
krFradSO_10 = p.CI.kfFradSO_10./Keq.FMNoxO2;
krFradSO_21 = p.CI.kfFradSO_21./Keq.FMNoxO2;
krFradSO_32 = p.CI.kfFradSO_32./Keq.FMNoxO2;
% H2O2 and FMNred
krH_20 = p.CI.kfH_20./Keq.FMNredH2O2;
krH_31 = p.CI.kfH_31./Keq.FMNredH2O2;
krH_31b = p.CI.kfH_31b./Keq.FMNredH2O2;
krH_42 = p.CI.kfH_42./Keq.FMNredH2O2;

% Nucleotide and pH effects
p.CI.kfH_20 = p.CI.kfH_20./PNr;
p.CI.kfH_42 = p.CI.kfH_42./PNr;
p.CI.kfH_31 = p.CI.kfH_31./PNr;
p.CI.kfH_31b = p.CI.kfH_31b./PNr;

p.CI.kfFrSO_21 = p.CI.kfFrSO_21./PNr./(1+Hn./10^-p.CI.FMNred.pK);
p.CI.kfFrSO_32 = p.CI.kfFrSO_32./PNr./(1+Hn./10^-p.CI.FMNred.pK);
p.CI.kfFrSO_32b = p.CI.kfFrSO_32b./PNr./(1+Hn./10^-p.CI.FMNred.pK);
p.CI.kfFrSO_43 = p.CI.kfFrSO_43./PNr./(1+Hn./10^-p.CI.FMNred.pK);

p.CI.kfFradSO_10 = p.CI.kfFradSO_10./PNrad./(1+Hn./10^-p.CI.FMNrad.pK);
p.CI.kfFradSO_21 = p.CI.kfFradSO_21./PNrad./(1+Hn./10^-p.CI.FMNrad.pK);
p.CI.kfFradSO_21b = p.CI.kfFradSO_21b./PNrad./(1+Hn./10^-p.CI.FMNrad.pK);
p.CI.kfFradSO_32 = p.CI.kfFradSO_32./PNrad./(1+Hn./10^-p.CI.FMNrad.pK);

p.CI.kfN2rSO_10 = p.CI.kfN2rSO_10./(1+Hn./10^-p.CI.N2.pKred);
p.CI.kfN2rSO_21 = p.CI.kfN2rSO_21./(1+Hn./10^-p.CI.N2.pKred);
p.CI.kfN2rSO_21b = p.CI.kfN2rSO_21b./(1+Hn./10^-p.CI.N2.pKred);
p.CI.kfN2rSO_32 = p.CI.kfN2rSO_32./(1+Hn./10^-p.CI.N2.pKred);
p.CI.kfN2rSO_32b = p.CI.kfN2rSO_32b./(1+Hn./10^-p.CI.N2.pKred);
p.CI.kfN2rSO_43 = p.CI.kfN2rSO_43./(1+Hn./10^-p.CI.N2.pKred);

% net forward rates

% membrane potential dependence
dPsiNf = 1;
dPsiNr = 1;
dPsiQf = exp(-4*p.CI.beta1*muH/p.R/p.T);
dPsiQr = exp(4*(1-p.CI.beta1)*muH/p.R/p.T);

% NADH transitions
kfNADH_02 = p.CI.kfNADH_02.*dPsiNf.*NADH./KdNADHo./PNo.*(s0.F_N2).*PH1.*PH2;
krNADH_02 = krN0_02.*dPsiNr.*NAD./KdNADr./PNr.*(s2.Fr_N2);
kfNADH_24 = p.CI.kfNADH_24.*dPsiNf.*NADH./KdNADHo./PNo.*(s2.F_N2r_SQ).*PH1.*PH2;
krNADH_24 = krN0_24.*dPsiNr.*NAD./KdNADr./PNr.*(s4.Fr_N2r_SQ);
kfNADH_13 = p.CI.kfNADH_13.*dPsiNf.*NADH./KdNADHo./PNo.*(s1.F_N2r + s1.F_N2_SQ).*PH1.*PH2;
krNADH_13 = krN0_13.*dPsiNr.*NAD./KdNADr./PNr.*(s3.Fr_N2r + s3.Fr_N2_SQ);

% QH2 transitions
kfQ_20 = p.CI.kfQ_20.*dPsiQf.*(s2.F_N2r_SQ).*PH3;
krQ_20 = krQ0_20.*dPsiQr.*QH2./KdQH2o./PQo.*(s0.F_N2);
kfQ_42 = p.CI.kfQ_42.*dPsiQf.*(s4.Fr_N2r_SQ).*PH3;
krQ_42 = krQ0_42.*dPsiQr.*QH2./KdQH2o./PQo.*(s2.Fr_N2);
kfQ_31 = p.CI.kfQ_31.*dPsiQf.*(s3.Frad_N2r_SQ).*PH3;
krQ_31 = krQ0_31.*dPsiQr.*QH2./KdQH2o./PQo.*(s1.Frad_N2);

% H2O2 transitions
kfH2O2_20 = p.CI.kfH_20.*O2.*s2.Fr_N2;
krH2O2_20 = krH_20.*H2O2.*s0.F_N2;

kfH2O2_31 = p.CI.kfH_31.*O2.*s3.Fr_N2r + p.CI.kfH_31b.*O2.*s3.Fr_N2_SQ;
krH2O2_31 = krH_31.*H2O2.*s1.F_N2r + krH_31b.*H2O2.*s1.F_N2_SQ;

kfH2O2_42 = p.CI.kfH_42.*O2.*s4.Fr_N2r_SQ;
krH2O2_42 = krH_42.*H2O2.*(s2.F_N2r_SQ);

% SO transitions
kfSO_10 = O2.*(p.CI.kfSQSO_10.*s1.F_N2_SQ + p.CI.kfFradSO_10.*s1.Frad_N2 + p.CI.kfN2rSO_10.*s1.F_N2r);
krSO_10 = SO.*s0.F_N2.*(krSQSO_10.*(Q./KdQo./PQo) + krFradSO_10 + krN2rSO_10);

kfSO_21 = O2.*((p.CI.kfSQSO_21.*s2.Frad_N2_SQ + p.CI.kfSQSO_21b.*s2.F_N2r_SQ) + p.CI.kfFrSO_21.*s2.Fr_N2 + (p.CI.kfFradSO_21.*s2.Frad_N2r + p.CI.kfFradSO_21b.*s2.Frad_N2_SQ) + (p.CI.kfN2rSO_21.*s2.Frad_N2r + p.CI.kfN2rSO_21b.*s2.F_N2r_SQ));
krSO_21 = SO.*(krFrSO_21.*s1.Frad_N2 + krN2rSO_21.*s1.Frad_N2 + krN2rSO_21b.*s1.F_N2_SQ + krFradSO_21.*s1.F_N2r + krFradSO_21b.*s1.F_N2_SQ + (krSQSO_21.*s1.Frad_N2.*(Q./KdQo./PQo) + krSQSO_21b.*s1.F_N2r.*(Q./KdQr./PQr)));

kfSO_32 = O2.*((p.CI.kfSQSO_32.*s3.Fr_N2_SQ + p.CI.kfSQSO_32b.*s3.Frad_N2r_SQ) + (p.CI.kfFrSO_32.*s3.Fr_N2r + p.CI.kfFrSO_32b.*s3.Fr_N2_SQ) + p.CI.kfFradSO_32.*s3.Frad_N2r_SQ + (p.CI.kfN2rSO_32.*s3.Fr_N2r + p.CI.kfN2rSO_32b.*s3.Frad_N2r_SQ));
krSO_32 = SO.*(krSQSO_32.*s2.Fr_N2.*(Q./KdQo./PQo) + krSQSO_32b.*s2.Frad_N2r.*(Q./KdQr./PQr) + krFradSO_32.*s2.F_N2r_SQ + krFrSO_32.*s2.Frad_N2r + krFrSO_32b.*s2.Frad_N2_SQ + krN2rSO_32.*s2.Fr_N2 + krN2rSO_32b.*s2.Frad_N2_SQ);

kfSO_43 = O2.*((p.CI.kfSQSO_43 + p.CI.kfN2rSO_43).*(s4.Fr_N2r_SQ)  + p.CI.kfFrSO_43.*s4.Fr_N2r_SQ);
krSO_43 = SO.*(krSQSO_43.*s3.Fr_N2r.*(Q./KdQr./PQr) + krFrSO_43.*s3.Frad_N2r_SQ  + krN2rSO_43.*s3.Fr_N2_SQ);
    
% S0 <-> S2
k0_2 = kfNADH_02 + krQ_20 + krH2O2_20;
k2_0 = kfQ_20 + krNADH_02 + kfH2O2_20; 

% S2 <-> S4
k2_4 = kfNADH_24 + krQ_42 + krH2O2_42;
k4_2 = kfQ_42 + krNADH_24 + kfH2O2_42;

% S1 <-> S3
k1_3 = kfNADH_13 + krQ_31 + krH2O2_31;
k3_1 = kfQ_31 + krNADH_13 + kfH2O2_31; 

% S0 <-> S1 (superoxide)
k1_0 = kfSO_10;
k0_1 = krSO_10;

% S2 <-> S1 (superoxide)
k2_1 = kfSO_21;
k1_2 = krSO_21; % if approximateing 3 state model set equal to 1e99

% S3 <-> S2 (superoxide)
k3_2 = kfSO_32;
k2_3 = krSO_32;

% S4 <-> S3 (superoxide)
k4_3 = kfSO_43;
k3_4 = krSO_43; % if approximateing 3 state model set equal to 1e99

% Steady-State Fractional Occupancies (solved analytically)
S0=(k1_0.*k2_0.*k3_1.*k4_2 + k1_0.*k2_0.*k3_1.*k4_3 + k1_0.*k2_0.*k3_2.*k4_2 + k1_0.*k2_1.*k3_1.*k4_2 + k1_0.*k2_0.*k3_2.*k4_3 + k1_0.*k2_1.*k3_1.*k4_3 + k1_0.*k2_1.*k3_2.*k4_2 + k1_2.*k2_0.*k3_1.*k4_2 + k1_0.*k2_0.*k3_4.*k4_2 + k1_0.*k2_1.*k3_2.*k4_3 + k1_0.*k2_3.*k3_1.*k4_2 + k1_2.*k2_0.*k3_1.*k4_3 + k1_2.*k2_0.*k3_2.*k4_2 + k1_0.*k2_1.*k3_4.*k4_2 + k1_0.*k2_3.*k3_1.*k4_3 + k1_2.*k2_0.*k3_2.*k4_3 + k1_3.*k2_0.*k3_2.*k4_2 + k1_0.*k2_4.*k3_1.*k4_3 + k1_2.*k2_0.*k3_4.*k4_2 + k1_3.*k2_0.*k3_2.*k4_3 + k1_3.*k2_0.*k3_4.*k4_2)./(k0_2.*k1_0.*k2_4.*k3_1 + k0_1.*k1_2.*k2_4.*k3_1 + k0_1.*k1_3.*k2_0.*k3_4 + k0_2.*k1_0.*k2_4.*k3_2 + k0_1.*k1_2.*k2_4.*k3_2 + k0_1.*k1_3.*k2_1.*k3_4 + k0_2.*k1_0.*k2_3.*k3_4 + k0_2.*k1_2.*k2_4.*k3_1 + k0_1.*k1_2.*k2_3.*k3_4 + k0_1.*k1_3.*k2_4.*k3_2 + k0_2.*k1_0.*k2_4.*k3_4 + k0_2.*k1_2.*k2_4.*k3_2 + k0_2.*k1_3.*k2_1.*k3_4 + k0_1.*k1_2.*k2_4.*k3_4 + k0_1.*k1_3.*k2_3.*k3_4 + k0_2.*k1_2.*k2_3.*k3_4 + k0_2.*k1_3.*k2_4.*k3_2 + k0_1.*k1_3.*k2_4.*k3_4 + k0_2.*k1_2.*k2_4.*k3_4 + k0_2.*k1_3.*k2_3.*k3_4 + k0_2.*k1_3.*k2_4.*k3_4 + k0_1.*k1_3.*k2_0.*k4_2 + k0_1.*k1_3.*k2_0.*k4_3 + k0_1.*k1_3.*k2_1.*k4_2 + k0_2.*k1_0.*k2_3.*k4_2 + k0_1.*k1_2.*k2_3.*k4_2 + k0_1.*k1_3.*k2_1.*k4_3 + k0_2.*k1_0.*k2_3.*k4_3 + k0_2.*k1_3.*k2_1.*k4_2 + k0_1.*k1_2.*k2_3.*k4_3 + k0_1.*k1_3.*k2_3.*k4_2 + k0_2.*k1_0.*k2_4.*k4_3 + k0_2.*k1_2.*k2_3.*k4_2 + k0_2.*k1_3.*k2_1.*k4_3 + k0_1.*k1_2.*k2_4.*k4_3 + k0_1.*k1_3.*k2_3.*k4_3 + k0_2.*k1_2.*k2_3.*k4_3 + k0_2.*k1_3.*k2_3.*k4_2 + k0_1.*k1_3.*k2_4.*k4_3 + k0_2.*k1_2.*k2_4.*k4_3 + k0_2.*k1_3.*k2_3.*k4_3 + k0_2.*k1_3.*k2_4.*k4_3 + k0_2.*k1_0.*k3_1.*k4_2 + k0_1.*k1_2.*k3_1.*k4_2 + k0_2.*k1_0.*k3_1.*k4_3 + k0_2.*k1_0.*k3_2.*k4_2 + k0_1.*k1_2.*k3_1.*k4_3 + k0_1.*k1_2.*k3_2.*k4_2 + k0_2.*k1_0.*k3_2.*k4_3 + k0_2.*k1_2.*k3_1.*k4_2 + k0_1.*k1_2.*k3_2.*k4_3 + k0_1.*k1_3.*k3_2.*k4_2 + k0_2.*k1_0.*k3_4.*k4_2 + k0_2.*k1_2.*k3_1.*k4_3 + k0_2.*k1_2.*k3_2.*k4_2 + k0_1.*k1_2.*k3_4.*k4_2 + k0_1.*k1_3.*k3_2.*k4_3 + k0_2.*k1_2.*k3_2.*k4_3 + k0_2.*k1_3.*k3_2.*k4_2 + k0_1.*k1_3.*k3_4.*k4_2 + k0_2.*k1_2.*k3_4.*k4_2 + k0_2.*k1_3.*k3_2.*k4_3 + k0_2.*k1_3.*k3_4.*k4_2 + k0_1.*k2_0.*k3_1.*k4_2 + k0_1.*k2_0.*k3_1.*k4_3 + k0_1.*k2_0.*k3_2.*k4_2 + k0_1.*k2_1.*k3_1.*k4_2 + k0_1.*k2_0.*k3_2.*k4_3 + k0_1.*k2_1.*k3_1.*k4_3 + k0_1.*k2_1.*k3_2.*k4_2 + k0_2.*k2_1.*k3_1.*k4_2 + k0_1.*k2_0.*k3_4.*k4_2 + k0_1.*k2_1.*k3_2.*k4_3 + k0_1.*k2_3.*k3_1.*k4_2 + k0_2.*k2_1.*k3_1.*k4_3 + k0_2.*k2_1.*k3_2.*k4_2 + k0_1.*k2_1.*k3_4.*k4_2 + k0_1.*k2_3.*k3_1.*k4_3 + k0_2.*k2_1.*k3_2.*k4_3 + k0_2.*k2_3.*k3_1.*k4_2 + k0_1.*k2_4.*k3_1.*k4_3 + k0_2.*k2_1.*k3_4.*k4_2 + k0_2.*k2_3.*k3_1.*k4_3 + k0_2.*k2_4.*k3_1.*k4_3 + k1_0.*k2_0.*k3_1.*k4_2 + k1_0.*k2_0.*k3_1.*k4_3 + k1_0.*k2_0.*k3_2.*k4_2 + k1_0.*k2_1.*k3_1.*k4_2 + k1_0.*k2_0.*k3_2.*k4_3 + k1_0.*k2_1.*k3_1.*k4_3 + k1_0.*k2_1.*k3_2.*k4_2 + k1_2.*k2_0.*k3_1.*k4_2 + k1_0.*k2_0.*k3_4.*k4_2 + k1_0.*k2_1.*k3_2.*k4_3 + k1_0.*k2_3.*k3_1.*k4_2 + k1_2.*k2_0.*k3_1.*k4_3 + k1_2.*k2_0.*k3_2.*k4_2 + k1_0.*k2_1.*k3_4.*k4_2 + k1_0.*k2_3.*k3_1.*k4_3 + k1_2.*k2_0.*k3_2.*k4_3 + k1_3.*k2_0.*k3_2.*k4_2 + k1_0.*k2_4.*k3_1.*k4_3 + k1_2.*k2_0.*k3_4.*k4_2 + k1_3.*k2_0.*k3_2.*k4_3 + k1_3.*k2_0.*k3_4.*k4_2);
S1=(k0_1.*k2_0.*k3_1.*k4_2 + k0_1.*k2_0.*k3_1.*k4_3 + k0_1.*k2_0.*k3_2.*k4_2 + k0_1.*k2_1.*k3_1.*k4_2 + k0_1.*k2_0.*k3_2.*k4_3 + k0_1.*k2_1.*k3_1.*k4_3 + k0_1.*k2_1.*k3_2.*k4_2 + k0_2.*k2_1.*k3_1.*k4_2 + k0_1.*k2_0.*k3_4.*k4_2 + k0_1.*k2_1.*k3_2.*k4_3 + k0_1.*k2_3.*k3_1.*k4_2 + k0_2.*k2_1.*k3_1.*k4_3 + k0_2.*k2_1.*k3_2.*k4_2 + k0_1.*k2_1.*k3_4.*k4_2 + k0_1.*k2_3.*k3_1.*k4_3 + k0_2.*k2_1.*k3_2.*k4_3 + k0_2.*k2_3.*k3_1.*k4_2 + k0_1.*k2_4.*k3_1.*k4_3 + k0_2.*k2_1.*k3_4.*k4_2 + k0_2.*k2_3.*k3_1.*k4_3 + k0_2.*k2_4.*k3_1.*k4_3)./(k0_2.*k1_0.*k2_4.*k3_1 + k0_1.*k1_2.*k2_4.*k3_1 + k0_1.*k1_3.*k2_0.*k3_4 + k0_2.*k1_0.*k2_4.*k3_2 + k0_1.*k1_2.*k2_4.*k3_2 + k0_1.*k1_3.*k2_1.*k3_4 + k0_2.*k1_0.*k2_3.*k3_4 + k0_2.*k1_2.*k2_4.*k3_1 + k0_1.*k1_2.*k2_3.*k3_4 + k0_1.*k1_3.*k2_4.*k3_2 + k0_2.*k1_0.*k2_4.*k3_4 + k0_2.*k1_2.*k2_4.*k3_2 + k0_2.*k1_3.*k2_1.*k3_4 + k0_1.*k1_2.*k2_4.*k3_4 + k0_1.*k1_3.*k2_3.*k3_4 + k0_2.*k1_2.*k2_3.*k3_4 + k0_2.*k1_3.*k2_4.*k3_2 + k0_1.*k1_3.*k2_4.*k3_4 + k0_2.*k1_2.*k2_4.*k3_4 + k0_2.*k1_3.*k2_3.*k3_4 + k0_2.*k1_3.*k2_4.*k3_4 + k0_1.*k1_3.*k2_0.*k4_2 + k0_1.*k1_3.*k2_0.*k4_3 + k0_1.*k1_3.*k2_1.*k4_2 + k0_2.*k1_0.*k2_3.*k4_2 + k0_1.*k1_2.*k2_3.*k4_2 + k0_1.*k1_3.*k2_1.*k4_3 + k0_2.*k1_0.*k2_3.*k4_3 + k0_2.*k1_3.*k2_1.*k4_2 + k0_1.*k1_2.*k2_3.*k4_3 + k0_1.*k1_3.*k2_3.*k4_2 + k0_2.*k1_0.*k2_4.*k4_3 + k0_2.*k1_2.*k2_3.*k4_2 + k0_2.*k1_3.*k2_1.*k4_3 + k0_1.*k1_2.*k2_4.*k4_3 + k0_1.*k1_3.*k2_3.*k4_3 + k0_2.*k1_2.*k2_3.*k4_3 + k0_2.*k1_3.*k2_3.*k4_2 + k0_1.*k1_3.*k2_4.*k4_3 + k0_2.*k1_2.*k2_4.*k4_3 + k0_2.*k1_3.*k2_3.*k4_3 + k0_2.*k1_3.*k2_4.*k4_3 + k0_2.*k1_0.*k3_1.*k4_2 + k0_1.*k1_2.*k3_1.*k4_2 + k0_2.*k1_0.*k3_1.*k4_3 + k0_2.*k1_0.*k3_2.*k4_2 + k0_1.*k1_2.*k3_1.*k4_3 + k0_1.*k1_2.*k3_2.*k4_2 + k0_2.*k1_0.*k3_2.*k4_3 + k0_2.*k1_2.*k3_1.*k4_2 + k0_1.*k1_2.*k3_2.*k4_3 + k0_1.*k1_3.*k3_2.*k4_2 + k0_2.*k1_0.*k3_4.*k4_2 + k0_2.*k1_2.*k3_1.*k4_3 + k0_2.*k1_2.*k3_2.*k4_2 + k0_1.*k1_2.*k3_4.*k4_2 + k0_1.*k1_3.*k3_2.*k4_3 + k0_2.*k1_2.*k3_2.*k4_3 + k0_2.*k1_3.*k3_2.*k4_2 + k0_1.*k1_3.*k3_4.*k4_2 + k0_2.*k1_2.*k3_4.*k4_2 + k0_2.*k1_3.*k3_2.*k4_3 + k0_2.*k1_3.*k3_4.*k4_2 + k0_1.*k2_0.*k3_1.*k4_2 + k0_1.*k2_0.*k3_1.*k4_3 + k0_1.*k2_0.*k3_2.*k4_2 + k0_1.*k2_1.*k3_1.*k4_2 + k0_1.*k2_0.*k3_2.*k4_3 + k0_1.*k2_1.*k3_1.*k4_3 + k0_1.*k2_1.*k3_2.*k4_2 + k0_2.*k2_1.*k3_1.*k4_2 + k0_1.*k2_0.*k3_4.*k4_2 + k0_1.*k2_1.*k3_2.*k4_3 + k0_1.*k2_3.*k3_1.*k4_2 + k0_2.*k2_1.*k3_1.*k4_3 + k0_2.*k2_1.*k3_2.*k4_2 + k0_1.*k2_1.*k3_4.*k4_2 + k0_1.*k2_3.*k3_1.*k4_3 + k0_2.*k2_1.*k3_2.*k4_3 + k0_2.*k2_3.*k3_1.*k4_2 + k0_1.*k2_4.*k3_1.*k4_3 + k0_2.*k2_1.*k3_4.*k4_2 + k0_2.*k2_3.*k3_1.*k4_3 + k0_2.*k2_4.*k3_1.*k4_3 + k1_0.*k2_0.*k3_1.*k4_2 + k1_0.*k2_0.*k3_1.*k4_3 + k1_0.*k2_0.*k3_2.*k4_2 + k1_0.*k2_1.*k3_1.*k4_2 + k1_0.*k2_0.*k3_2.*k4_3 + k1_0.*k2_1.*k3_1.*k4_3 + k1_0.*k2_1.*k3_2.*k4_2 + k1_2.*k2_0.*k3_1.*k4_2 + k1_0.*k2_0.*k3_4.*k4_2 + k1_0.*k2_1.*k3_2.*k4_3 + k1_0.*k2_3.*k3_1.*k4_2 + k1_2.*k2_0.*k3_1.*k4_3 + k1_2.*k2_0.*k3_2.*k4_2 + k1_0.*k2_1.*k3_4.*k4_2 + k1_0.*k2_3.*k3_1.*k4_3 + k1_2.*k2_0.*k3_2.*k4_3 + k1_3.*k2_0.*k3_2.*k4_2 + k1_0.*k2_4.*k3_1.*k4_3 + k1_2.*k2_0.*k3_4.*k4_2 + k1_3.*k2_0.*k3_2.*k4_3 + k1_3.*k2_0.*k3_4.*k4_2);
S2=(k0_2.*k1_0.*k3_1.*k4_2 + k0_1.*k1_2.*k3_1.*k4_2 + k0_2.*k1_0.*k3_1.*k4_3 + k0_2.*k1_0.*k3_2.*k4_2 + k0_1.*k1_2.*k3_1.*k4_3 + k0_1.*k1_2.*k3_2.*k4_2 + k0_2.*k1_0.*k3_2.*k4_3 + k0_2.*k1_2.*k3_1.*k4_2 + k0_1.*k1_2.*k3_2.*k4_3 + k0_1.*k1_3.*k3_2.*k4_2 + k0_2.*k1_0.*k3_4.*k4_2 + k0_2.*k1_2.*k3_1.*k4_3 + k0_2.*k1_2.*k3_2.*k4_2 + k0_1.*k1_2.*k3_4.*k4_2 + k0_1.*k1_3.*k3_2.*k4_3 + k0_2.*k1_2.*k3_2.*k4_3 + k0_2.*k1_3.*k3_2.*k4_2 + k0_1.*k1_3.*k3_4.*k4_2 + k0_2.*k1_2.*k3_4.*k4_2 + k0_2.*k1_3.*k3_2.*k4_3 + k0_2.*k1_3.*k3_4.*k4_2)./(k0_2.*k1_0.*k2_4.*k3_1 + k0_1.*k1_2.*k2_4.*k3_1 + k0_1.*k1_3.*k2_0.*k3_4 + k0_2.*k1_0.*k2_4.*k3_2 + k0_1.*k1_2.*k2_4.*k3_2 + k0_1.*k1_3.*k2_1.*k3_4 + k0_2.*k1_0.*k2_3.*k3_4 + k0_2.*k1_2.*k2_4.*k3_1 + k0_1.*k1_2.*k2_3.*k3_4 + k0_1.*k1_3.*k2_4.*k3_2 + k0_2.*k1_0.*k2_4.*k3_4 + k0_2.*k1_2.*k2_4.*k3_2 + k0_2.*k1_3.*k2_1.*k3_4 + k0_1.*k1_2.*k2_4.*k3_4 + k0_1.*k1_3.*k2_3.*k3_4 + k0_2.*k1_2.*k2_3.*k3_4 + k0_2.*k1_3.*k2_4.*k3_2 + k0_1.*k1_3.*k2_4.*k3_4 + k0_2.*k1_2.*k2_4.*k3_4 + k0_2.*k1_3.*k2_3.*k3_4 + k0_2.*k1_3.*k2_4.*k3_4 + k0_1.*k1_3.*k2_0.*k4_2 + k0_1.*k1_3.*k2_0.*k4_3 + k0_1.*k1_3.*k2_1.*k4_2 + k0_2.*k1_0.*k2_3.*k4_2 + k0_1.*k1_2.*k2_3.*k4_2 + k0_1.*k1_3.*k2_1.*k4_3 + k0_2.*k1_0.*k2_3.*k4_3 + k0_2.*k1_3.*k2_1.*k4_2 + k0_1.*k1_2.*k2_3.*k4_3 + k0_1.*k1_3.*k2_3.*k4_2 + k0_2.*k1_0.*k2_4.*k4_3 + k0_2.*k1_2.*k2_3.*k4_2 + k0_2.*k1_3.*k2_1.*k4_3 + k0_1.*k1_2.*k2_4.*k4_3 + k0_1.*k1_3.*k2_3.*k4_3 + k0_2.*k1_2.*k2_3.*k4_3 + k0_2.*k1_3.*k2_3.*k4_2 + k0_1.*k1_3.*k2_4.*k4_3 + k0_2.*k1_2.*k2_4.*k4_3 + k0_2.*k1_3.*k2_3.*k4_3 + k0_2.*k1_3.*k2_4.*k4_3 + k0_2.*k1_0.*k3_1.*k4_2 + k0_1.*k1_2.*k3_1.*k4_2 + k0_2.*k1_0.*k3_1.*k4_3 + k0_2.*k1_0.*k3_2.*k4_2 + k0_1.*k1_2.*k3_1.*k4_3 + k0_1.*k1_2.*k3_2.*k4_2 + k0_2.*k1_0.*k3_2.*k4_3 + k0_2.*k1_2.*k3_1.*k4_2 + k0_1.*k1_2.*k3_2.*k4_3 + k0_1.*k1_3.*k3_2.*k4_2 + k0_2.*k1_0.*k3_4.*k4_2 + k0_2.*k1_2.*k3_1.*k4_3 + k0_2.*k1_2.*k3_2.*k4_2 + k0_1.*k1_2.*k3_4.*k4_2 + k0_1.*k1_3.*k3_2.*k4_3 + k0_2.*k1_2.*k3_2.*k4_3 + k0_2.*k1_3.*k3_2.*k4_2 + k0_1.*k1_3.*k3_4.*k4_2 + k0_2.*k1_2.*k3_4.*k4_2 + k0_2.*k1_3.*k3_2.*k4_3 + k0_2.*k1_3.*k3_4.*k4_2 + k0_1.*k2_0.*k3_1.*k4_2 + k0_1.*k2_0.*k3_1.*k4_3 + k0_1.*k2_0.*k3_2.*k4_2 + k0_1.*k2_1.*k3_1.*k4_2 + k0_1.*k2_0.*k3_2.*k4_3 + k0_1.*k2_1.*k3_1.*k4_3 + k0_1.*k2_1.*k3_2.*k4_2 + k0_2.*k2_1.*k3_1.*k4_2 + k0_1.*k2_0.*k3_4.*k4_2 + k0_1.*k2_1.*k3_2.*k4_3 + k0_1.*k2_3.*k3_1.*k4_2 + k0_2.*k2_1.*k3_1.*k4_3 + k0_2.*k2_1.*k3_2.*k4_2 + k0_1.*k2_1.*k3_4.*k4_2 + k0_1.*k2_3.*k3_1.*k4_3 + k0_2.*k2_1.*k3_2.*k4_3 + k0_2.*k2_3.*k3_1.*k4_2 + k0_1.*k2_4.*k3_1.*k4_3 + k0_2.*k2_1.*k3_4.*k4_2 + k0_2.*k2_3.*k3_1.*k4_3 + k0_2.*k2_4.*k3_1.*k4_3 + k1_0.*k2_0.*k3_1.*k4_2 + k1_0.*k2_0.*k3_1.*k4_3 + k1_0.*k2_0.*k3_2.*k4_2 + k1_0.*k2_1.*k3_1.*k4_2 + k1_0.*k2_0.*k3_2.*k4_3 + k1_0.*k2_1.*k3_1.*k4_3 + k1_0.*k2_1.*k3_2.*k4_2 + k1_2.*k2_0.*k3_1.*k4_2 + k1_0.*k2_0.*k3_4.*k4_2 + k1_0.*k2_1.*k3_2.*k4_3 + k1_0.*k2_3.*k3_1.*k4_2 + k1_2.*k2_0.*k3_1.*k4_3 + k1_2.*k2_0.*k3_2.*k4_2 + k1_0.*k2_1.*k3_4.*k4_2 + k1_0.*k2_3.*k3_1.*k4_3 + k1_2.*k2_0.*k3_2.*k4_3 + k1_3.*k2_0.*k3_2.*k4_2 + k1_0.*k2_4.*k3_1.*k4_3 + k1_2.*k2_0.*k3_4.*k4_2 + k1_3.*k2_0.*k3_2.*k4_3 + k1_3.*k2_0.*k3_4.*k4_2);
S3=(k0_1.*k1_3.*k2_0.*k4_2 + k0_1.*k1_3.*k2_0.*k4_3 + k0_1.*k1_3.*k2_1.*k4_2 + k0_2.*k1_0.*k2_3.*k4_2 + k0_1.*k1_2.*k2_3.*k4_2 + k0_1.*k1_3.*k2_1.*k4_3 + k0_2.*k1_0.*k2_3.*k4_3 + k0_2.*k1_3.*k2_1.*k4_2 + k0_1.*k1_2.*k2_3.*k4_3 + k0_1.*k1_3.*k2_3.*k4_2 + k0_2.*k1_0.*k2_4.*k4_3 + k0_2.*k1_2.*k2_3.*k4_2 + k0_2.*k1_3.*k2_1.*k4_3 + k0_1.*k1_2.*k2_4.*k4_3 + k0_1.*k1_3.*k2_3.*k4_3 + k0_2.*k1_2.*k2_3.*k4_3 + k0_2.*k1_3.*k2_3.*k4_2 + k0_1.*k1_3.*k2_4.*k4_3 + k0_2.*k1_2.*k2_4.*k4_3 + k0_2.*k1_3.*k2_3.*k4_3 + k0_2.*k1_3.*k2_4.*k4_3)./(k0_2.*k1_0.*k2_4.*k3_1 + k0_1.*k1_2.*k2_4.*k3_1 + k0_1.*k1_3.*k2_0.*k3_4 + k0_2.*k1_0.*k2_4.*k3_2 + k0_1.*k1_2.*k2_4.*k3_2 + k0_1.*k1_3.*k2_1.*k3_4 + k0_2.*k1_0.*k2_3.*k3_4 + k0_2.*k1_2.*k2_4.*k3_1 + k0_1.*k1_2.*k2_3.*k3_4 + k0_1.*k1_3.*k2_4.*k3_2 + k0_2.*k1_0.*k2_4.*k3_4 + k0_2.*k1_2.*k2_4.*k3_2 + k0_2.*k1_3.*k2_1.*k3_4 + k0_1.*k1_2.*k2_4.*k3_4 + k0_1.*k1_3.*k2_3.*k3_4 + k0_2.*k1_2.*k2_3.*k3_4 + k0_2.*k1_3.*k2_4.*k3_2 + k0_1.*k1_3.*k2_4.*k3_4 + k0_2.*k1_2.*k2_4.*k3_4 + k0_2.*k1_3.*k2_3.*k3_4 + k0_2.*k1_3.*k2_4.*k3_4 + k0_1.*k1_3.*k2_0.*k4_2 + k0_1.*k1_3.*k2_0.*k4_3 + k0_1.*k1_3.*k2_1.*k4_2 + k0_2.*k1_0.*k2_3.*k4_2 + k0_1.*k1_2.*k2_3.*k4_2 + k0_1.*k1_3.*k2_1.*k4_3 + k0_2.*k1_0.*k2_3.*k4_3 + k0_2.*k1_3.*k2_1.*k4_2 + k0_1.*k1_2.*k2_3.*k4_3 + k0_1.*k1_3.*k2_3.*k4_2 + k0_2.*k1_0.*k2_4.*k4_3 + k0_2.*k1_2.*k2_3.*k4_2 + k0_2.*k1_3.*k2_1.*k4_3 + k0_1.*k1_2.*k2_4.*k4_3 + k0_1.*k1_3.*k2_3.*k4_3 + k0_2.*k1_2.*k2_3.*k4_3 + k0_2.*k1_3.*k2_3.*k4_2 + k0_1.*k1_3.*k2_4.*k4_3 + k0_2.*k1_2.*k2_4.*k4_3 + k0_2.*k1_3.*k2_3.*k4_3 + k0_2.*k1_3.*k2_4.*k4_3 + k0_2.*k1_0.*k3_1.*k4_2 + k0_1.*k1_2.*k3_1.*k4_2 + k0_2.*k1_0.*k3_1.*k4_3 + k0_2.*k1_0.*k3_2.*k4_2 + k0_1.*k1_2.*k3_1.*k4_3 + k0_1.*k1_2.*k3_2.*k4_2 + k0_2.*k1_0.*k3_2.*k4_3 + k0_2.*k1_2.*k3_1.*k4_2 + k0_1.*k1_2.*k3_2.*k4_3 + k0_1.*k1_3.*k3_2.*k4_2 + k0_2.*k1_0.*k3_4.*k4_2 + k0_2.*k1_2.*k3_1.*k4_3 + k0_2.*k1_2.*k3_2.*k4_2 + k0_1.*k1_2.*k3_4.*k4_2 + k0_1.*k1_3.*k3_2.*k4_3 + k0_2.*k1_2.*k3_2.*k4_3 + k0_2.*k1_3.*k3_2.*k4_2 + k0_1.*k1_3.*k3_4.*k4_2 + k0_2.*k1_2.*k3_4.*k4_2 + k0_2.*k1_3.*k3_2.*k4_3 + k0_2.*k1_3.*k3_4.*k4_2 + k0_1.*k2_0.*k3_1.*k4_2 + k0_1.*k2_0.*k3_1.*k4_3 + k0_1.*k2_0.*k3_2.*k4_2 + k0_1.*k2_1.*k3_1.*k4_2 + k0_1.*k2_0.*k3_2.*k4_3 + k0_1.*k2_1.*k3_1.*k4_3 + k0_1.*k2_1.*k3_2.*k4_2 + k0_2.*k2_1.*k3_1.*k4_2 + k0_1.*k2_0.*k3_4.*k4_2 + k0_1.*k2_1.*k3_2.*k4_3 + k0_1.*k2_3.*k3_1.*k4_2 + k0_2.*k2_1.*k3_1.*k4_3 + k0_2.*k2_1.*k3_2.*k4_2 + k0_1.*k2_1.*k3_4.*k4_2 + k0_1.*k2_3.*k3_1.*k4_3 + k0_2.*k2_1.*k3_2.*k4_3 + k0_2.*k2_3.*k3_1.*k4_2 + k0_1.*k2_4.*k3_1.*k4_3 + k0_2.*k2_1.*k3_4.*k4_2 + k0_2.*k2_3.*k3_1.*k4_3 + k0_2.*k2_4.*k3_1.*k4_3 + k1_0.*k2_0.*k3_1.*k4_2 + k1_0.*k2_0.*k3_1.*k4_3 + k1_0.*k2_0.*k3_2.*k4_2 + k1_0.*k2_1.*k3_1.*k4_2 + k1_0.*k2_0.*k3_2.*k4_3 + k1_0.*k2_1.*k3_1.*k4_3 + k1_0.*k2_1.*k3_2.*k4_2 + k1_2.*k2_0.*k3_1.*k4_2 + k1_0.*k2_0.*k3_4.*k4_2 + k1_0.*k2_1.*k3_2.*k4_3 + k1_0.*k2_3.*k3_1.*k4_2 + k1_2.*k2_0.*k3_1.*k4_3 + k1_2.*k2_0.*k3_2.*k4_2 + k1_0.*k2_1.*k3_4.*k4_2 + k1_0.*k2_3.*k3_1.*k4_3 + k1_2.*k2_0.*k3_2.*k4_3 + k1_3.*k2_0.*k3_2.*k4_2 + k1_0.*k2_4.*k3_1.*k4_3 + k1_2.*k2_0.*k3_4.*k4_2 + k1_3.*k2_0.*k3_2.*k4_3 + k1_3.*k2_0.*k3_4.*k4_2);
% S4=(k0_2.*k1_0.*k2_4.*k3_1 + k0_1.*k1_2.*k2_4.*k3_1 + k0_1.*k1_3.*k2_0.*k3_4 + k0_2.*k1_0.*k2_4.*k3_2 + k0_1.*k1_2.*k2_4.*k3_2 + k0_1.*k1_3.*k2_1.*k3_4 + k0_2.*k1_0.*k2_3.*k3_4 + k0_2.*k1_2.*k2_4.*k3_1 + k0_1.*k1_2.*k2_3.*k3_4 + k0_1.*k1_3.*k2_4.*k3_2 + k0_2.*k1_0.*k2_4.*k3_4 + k0_2.*k1_2.*k2_4.*k3_2 + k0_2.*k1_3.*k2_1.*k3_4 + k0_1.*k1_2.*k2_4.*k3_4 + k0_1.*k1_3.*k2_3.*k3_4 + k0_2.*k1_2.*k2_3.*k3_4 + k0_2.*k1_3.*k2_4.*k3_2 + k0_1.*k1_3.*k2_4.*k3_4 + k0_2.*k1_2.*k2_4.*k3_4 + k0_2.*k1_3.*k2_3.*k3_4 + k0_2.*k1_3.*k2_4.*k3_4)./(k0_2.*k1_0.*k2_4.*k3_1 + k0_1.*k1_2.*k2_4.*k3_1 + k0_1.*k1_3.*k2_0.*k3_4 + k0_2.*k1_0.*k2_4.*k3_2 + k0_1.*k1_2.*k2_4.*k3_2 + k0_1.*k1_3.*k2_1.*k3_4 + k0_2.*k1_0.*k2_3.*k3_4 + k0_2.*k1_2.*k2_4.*k3_1 + k0_1.*k1_2.*k2_3.*k3_4 + k0_1.*k1_3.*k2_4.*k3_2 + k0_2.*k1_0.*k2_4.*k3_4 + k0_2.*k1_2.*k2_4.*k3_2 + k0_2.*k1_3.*k2_1.*k3_4 + k0_1.*k1_2.*k2_4.*k3_4 + k0_1.*k1_3.*k2_3.*k3_4 + k0_2.*k1_2.*k2_3.*k3_4 + k0_2.*k1_3.*k2_4.*k3_2 + k0_1.*k1_3.*k2_4.*k3_4 + k0_2.*k1_2.*k2_4.*k3_4 + k0_2.*k1_3.*k2_3.*k3_4 + k0_2.*k1_3.*k2_4.*k3_4 + k0_1.*k1_3.*k2_0.*k4_2 + k0_1.*k1_3.*k2_0.*k4_3 + k0_1.*k1_3.*k2_1.*k4_2 + k0_2.*k1_0.*k2_3.*k4_2 + k0_1.*k1_2.*k2_3.*k4_2 + k0_1.*k1_3.*k2_1.*k4_3 + k0_2.*k1_0.*k2_3.*k4_3 + k0_2.*k1_3.*k2_1.*k4_2 + k0_1.*k1_2.*k2_3.*k4_3 + k0_1.*k1_3.*k2_3.*k4_2 + k0_2.*k1_0.*k2_4.*k4_3 + k0_2.*k1_2.*k2_3.*k4_2 + k0_2.*k1_3.*k2_1.*k4_3 + k0_1.*k1_2.*k2_4.*k4_3 + k0_1.*k1_3.*k2_3.*k4_3 + k0_2.*k1_2.*k2_3.*k4_3 + k0_2.*k1_3.*k2_3.*k4_2 + k0_1.*k1_3.*k2_4.*k4_3 + k0_2.*k1_2.*k2_4.*k4_3 + k0_2.*k1_3.*k2_3.*k4_3 + k0_2.*k1_3.*k2_4.*k4_3 + k0_2.*k1_0.*k3_1.*k4_2 + k0_1.*k1_2.*k3_1.*k4_2 + k0_2.*k1_0.*k3_1.*k4_3 + k0_2.*k1_0.*k3_2.*k4_2 + k0_1.*k1_2.*k3_1.*k4_3 + k0_1.*k1_2.*k3_2.*k4_2 + k0_2.*k1_0.*k3_2.*k4_3 + k0_2.*k1_2.*k3_1.*k4_2 + k0_1.*k1_2.*k3_2.*k4_3 + k0_1.*k1_3.*k3_2.*k4_2 + k0_2.*k1_0.*k3_4.*k4_2 + k0_2.*k1_2.*k3_1.*k4_3 + k0_2.*k1_2.*k3_2.*k4_2 + k0_1.*k1_2.*k3_4.*k4_2 + k0_1.*k1_3.*k3_2.*k4_3 + k0_2.*k1_2.*k3_2.*k4_3 + k0_2.*k1_3.*k3_2.*k4_2 + k0_1.*k1_3.*k3_4.*k4_2 + k0_2.*k1_2.*k3_4.*k4_2 + k0_2.*k1_3.*k3_2.*k4_3 + k0_2.*k1_3.*k3_4.*k4_2 + k0_1.*k2_0.*k3_1.*k4_2 + k0_1.*k2_0.*k3_1.*k4_3 + k0_1.*k2_0.*k3_2.*k4_2 + k0_1.*k2_1.*k3_1.*k4_2 + k0_1.*k2_0.*k3_2.*k4_3 + k0_1.*k2_1.*k3_1.*k4_3 + k0_1.*k2_1.*k3_2.*k4_2 + k0_2.*k2_1.*k3_1.*k4_2 + k0_1.*k2_0.*k3_4.*k4_2 + k0_1.*k2_1.*k3_2.*k4_3 + k0_1.*k2_3.*k3_1.*k4_2 + k0_2.*k2_1.*k3_1.*k4_3 + k0_2.*k2_1.*k3_2.*k4_2 + k0_1.*k2_1.*k3_4.*k4_2 + k0_1.*k2_3.*k3_1.*k4_3 + k0_2.*k2_1.*k3_2.*k4_3 + k0_2.*k2_3.*k3_1.*k4_2 + k0_1.*k2_4.*k3_1.*k4_3 + k0_2.*k2_1.*k3_4.*k4_2 + k0_2.*k2_3.*k3_1.*k4_3 + k0_2.*k2_4.*k3_1.*k4_3 + k1_0.*k2_0.*k3_1.*k4_2 + k1_0.*k2_0.*k3_1.*k4_3 + k1_0.*k2_0.*k3_2.*k4_2 + k1_0.*k2_1.*k3_1.*k4_2 + k1_0.*k2_0.*k3_2.*k4_3 + k1_0.*k2_1.*k3_1.*k4_3 + k1_0.*k2_1.*k3_2.*k4_2 + k1_2.*k2_0.*k3_1.*k4_2 + k1_0.*k2_0.*k3_4.*k4_2 + k1_0.*k2_1.*k3_2.*k4_3 + k1_0.*k2_3.*k3_1.*k4_2 + k1_2.*k2_0.*k3_1.*k4_3 + k1_2.*k2_0.*k3_2.*k4_2 + k1_0.*k2_1.*k3_4.*k4_2 + k1_0.*k2_3.*k3_1.*k4_3 + k1_2.*k2_0.*k3_2.*k4_3 + k1_3.*k2_0.*k3_2.*k4_2 + k1_0.*k2_4.*k3_1.*k4_3 + k1_2.*k2_0.*k3_4.*k4_2 + k1_3.*k2_0.*k3_2.*k4_3 + k1_3.*k2_0.*k3_4.*k4_2);
S4 = 1 - S0 - S1 - S2 - S3;

% Turnover Rate
JNADH = p.CI.x.*(kfNADH_02.*S0 - krNADH_02.*S2 ...
    + kfNADH_24.*S2 - krNADH_24.*S4 ...
    + kfNADH_13.*S1 - krNADH_13.*S3); % NADH oxidation releases 1 proton in matrix space
JSO = p.CI.x.*(S1.*(kfSO_10 - krSO_21) + S2.*(kfSO_21 - krSO_32) + S3.*(kfSO_32 - krSO_43) + S4.*kfSO_43 - krSO_10.*S0);
JH2O2 = p.CI.x.*(S2.*(kfH2O2_20 - krH2O2_42) + S3.*kfH2O2_31 + S4.*kfH2O2_42 - krH2O2_31.*S1 - krH2O2_20.*S0);
% JQ = p.CI.x.*(kfQ_20.*S2 - krQ_20.*S0 ...
%     + kfQ_42.*S4 - krQ_42.*S2 ...
%     + kfQ_31.*S3 - krQ_31.*S1); % charge transfer and proton pumping is the same rate as JQ, 4 charges and protons moved to IMS

% for numerical stability, enforce strict stoichiometric coupling
JQ = JNADH - JSO/2 - JH2O2;

% Output
varargout = {JNADH,JQ,JSO,JH2O2};
end

%% Assume SDH rate is proportional to JDH rate
function J = ETC2m0(x,p,JDH)
J = p.CII.x.*JDH;
end

%% This version is used for the ischemia/reperfusion analysis
function J = ETC2m1(x,p)
J = p.CII.x.*(1 - x(p.i.UQH2_x,:)./x(p.i.UQ_x,:)/p.CII.Keq);
end

%% Complex III Rate Equation (SQo-state is buried in E2 and E4 states)
%
% 6 State Model
% E0 - 0e- at Qo-site AND 0e- at Qi-site
% E1 - 1e- at Qo-site AND 0e- at Qi-site
% E2 - 1e- at Qi-site AND 0e- at Qi-site
% E3 - 1e- at Qo-site AND 1e- at Qi-site
% E4 - 2e- at Qo-site AND 1e- at Qi-site
% E5 - 2e- at Qo-site AND 0e- at Qi-site
function varargout = ETC3(x,p)
% define state variables
dPsi = x(p.i.dPsi,:);
Hp = x(p.i.H_c,:);
Hn = x(p.i.H_x,:);
Q = x(p.i.UQ_x,:);
% QH2 = p.CIII.Qtot - Q; % pool conserved in mitochondrial system
QH2 = x(p.i.UQH2_x,:);
c3 = x(p.i.c3_i,:);
% c2 = p.CIII.ctot - c3; % pool conserved in mitochondrial system
c2 = x(p.i.c2_i,:);
O2 = x(p.i.O2_x,:);
SO = (x(p.i.SO_x,:)+x(p.i.SO_c,:))/2;

% Set KDQH2 at Qi site
p.CIII.KQH2i = p.CIII.KQH2o^2*p.CIII.KQi*p.CIII.Kc3^2/p.CIII.KQo^2/p.CIII.Kc2^2; % MR constraint

% Update midpoint potentials from thermodynamic data
p.CIII.Em0.Q_QH2 = (p.dGf(p.if.UQH2) - p.dGf(p.if.UQ))/(-2*p.F);
p.CIII.Em0.c = (p.dGf(p.if.c2) - p.dGf(p.if.c3))/(-1*p.F);
p.CIII.Em0.SO = (p.dGf(p.if.SO) - p.dGf(p.if.O2))/(-1*p.F);

% Update Q Thermodynamics
p.CIII.Em0.Q_SQo = p.CIII.Em0.Q_QH2 + p.R*p.T/p.F/2*log(p.CIII.Kstabo*1e-14); % mV
p.CIII.Em0.SQ_QH2o = 2*p.CIII.Em0.Q_QH2 - p.CIII.Em0.Q_SQo; % mV (200 - 300)
p.CIII.Em0.Q_SQi = p.CIII.Em0.Q_QH2 + p.R*p.T/p.F/2*log(p.CIII.Kstabi*1e-14); % mV (assumes Kstabi at pH 7)
p.CIII.Em0.SQ_QH2i = 2*p.CIII.Em0.Q_QH2 - p.CIII.Em0.Q_SQi; % mV (16-150)

% Binding Polynomials for Protonated States
P_ISP = (1 + Hp/10^-p.CIII.pK.ISPox2 + Hp.^2/10^-p.CIII.pK.ISPox2/10^-p.CIII.pK.ISPox1);
P_bLox = (1 + Hp/10^-p.CIII.pK.bLox);
P_bLred = (1 + Hp/10^-p.CIII.pK.bLred);
P_bHox = (1 + Hn/10^-p.CIII.pK.bHox);
P_bHred = (1 + Hn/10^-p.CIII.pK.bHred);
P_QH2o = (1 + Hp./10^-p.CIII.pK.QH + Hp.^2./10^-p.CIII.pK.QH./10^-p.CIII.pK.QH2);
P_QH2i = (1 + Hn./10^-p.CIII.pK.QH + Hn.^2./10^-p.CIII.pK.QH./10^-p.CIII.pK.QH2);
P_SO = (1 + Hp./10^-p.CIII.pK.SO);

% Binding Polynomials for enzyme, substrates, products and regulators
% Qo-site
P_Qo = (1 + Q/p.CIII.KQo + QH2/p.CIII.KQH2o);
% Cytc c-site
P_c = (1 + c2/p.CIII.Kc2 + c3/p.CIII.Kc3);
% Qi-site
P_Qi = (1 + Q/p.CIII.KQi + QH2/p.CIII.KQH2i + p.CIII.AA/p.CIII.KAA);

% Midpoint potentials
% Qo-site
Em.c = p.CIII.Em0.c; % pH independent
Em.Q_QH2o = p.CIII.Em0.Q_QH2 - log(10)*p.R*p.T/p.F/2*log10(Hp.^2/10^-p.CIII.pK.QH2/10^-p.CIII.pK.QH./P_QH2o./Hp.^2); % assuming linked to P-side
Em.Q_SQo = p.CIII.Em0.Q_SQo; % pH independent
Em.SQ_QH2o = p.CIII.Em0.SQ_QH2o - log(10)*p.R*p.T/p.F*log10((Hp.^2/10^-p.CIII.pK.QH2/10^-p.CIII.pK.QH)./P_QH2o./Hp.^2); % assuming linked to P-side
Em.ISP = p.CIII.Em0.ISP - log(10)*p.R*p.T/p.F*log10(P_ISP./(Hp.^2/10^-p.CIII.pK.ISPox2/10^-p.CIII.pK.ISPox1)); % assuming linked to P-side 
Em.bL = p.CIII.Em0.bL - log(10)*p.R*p.T/p.F*log10((Hp+10^-p.CIII.pK.bLox)./(Hp+10^-p.CIII.pK.bLred)); % assuming linked to P-side (Izrailev et al. 1999, also Crofts), log10((Hp./10^-p.CIII.pK.bLred./P_bLred)./(Hp./10^-p.CIII.pK.bLox./P_bLox))

% Qi-site
Em.bH = p.CIII.Em0.bH - log(10)*p.R*p.T/p.F*log10((Hn+10^-p.CIII.pK.bHox)./(Hn+10^-p.CIII.pK.bHred)); % assuming linked to N-side, log10((Hn./10^-p.CIII.pK.bHred./P_bHred)./(Hn./10^-p.CIII.pK.bHox./P_bHox))
Em.Q_QH2i = p.CIII.Em0.Q_QH2 - log(10)*p.R*p.T/p.F/2*log10(Hn.^2/10^-p.CIII.pK.QH2/10^-p.CIII.pK.QH./P_QH2i./Hn.^2); % assuming linked to N-siden
Em.Q_SQi = p.CIII.Em0.Q_SQi; % pH independent
Em.SQ_QH2i = p.CIII.Em0.SQ_QH2i - log(10)*p.R*p.T/p.F*log10((Hn.^2/10^-p.CIII.pK.QH2/10^-p.CIII.pK.QH)./P_QH2i./Hn.^2); % assuming linked to N-side

% Superoxide
Em.SO = p.CIII.Em0.SO - log(10)*p.R*p.T/p.F*log10(1./P_SO/10^-p.CIII.pK.SO);

% State Transition Thermodynamics

% QH2o<->SQo
dGo = -p.F*(Em.c - Em.SQ_QH2o);
% dG1 = dGo + p.R*p.T*log((E2.*Q.*c2.*Hp.^2)./(QH2.*c3.*E1*1e-14).*(PE1.*p.CIII.KQH2o.*p.CIII.Kc3)./(PE2.*p.CIII.KQo.*p.CIII.Kc2));

% bHred<->bLred
dGi = -p.F*(Em.bH - Em.bL);
% dG2 = dGi + p.R*p.T*log((E3)./(E2.*Q).*(PE2*p.CIII.KQi)./(PE3)) + p.F*dPsi;

% Substates via Rapid Equilibrium

% ISP protonation state
PISP = Hp./10^-p.CIII.pK.ISPox2.*(1+Hp./10^-p.CIII.pK.ISPox1)./P_ISP;

% bL reduction by SQo
dGQo = -p.F*(Em.bL - Em.Q_SQo);
KQo = exp(-dGQo/p.R/p.T); % Q-bLred / SQ-bLox
% State 2 fractional species polynomial
% fQoa = 1; % SQo
rQo = KQo.*P_Qo*p.CIII.KQo./Q; % bL_red
PQo = 1 + rQo; % SQo

% Q reduction by bHred
dGQi = -p.F*(Em.Q_SQi - Em.bH);
KQi = exp(-dGQi/p.R/p.T); % bLox-SQ / bLred-Q
% State 3 fractional species polynomial
% fQia = 1; % bHred
rQi = KQi.*Q/p.CIII.KQi./P_Qi; % bHox-SQi
PQi = 1 + rQi; % bHred

% SQi reduction by bHred
dGQi2 = -p.F*(Em.SQ_QH2i - Em.bH) + p.CIII.beta2*2*p.F*dPsi; % protons included in Em.SQ_QH2i
KQi2 = exp(-dGQi2/p.R/p.T);
% fQi2a = 1; % bHred-SQi
rQi2 = KQi2.*P_Qi*p.CIII.KQH2i./QH2; % bHox
PQi2 = 1 + rQi2; %bHred-SQi

% Superoxide production/consumption rates
% SO thermodynamics
dG_SO = -p.F*(Em.SO - Em.Q_SQo);
kSOr = p.CIII.kSO*exp(dG_SO/p.R/p.T);

% Reverse Rate Constants

k210 = p.CIII.k120*exp(dGo/p.R/p.T);
k320 = p.CIII.k230*exp(dGi/p.R/p.T);
k430 = p.CIII.k340*exp(dGo/p.R/p.T);
k140 = p.CIII.k410*exp(dGi/p.R/p.T);
k540 = p.CIII.k450*exp(dGo/p.R/p.T);
k250 = p.CIII.k520*exp(dGi/p.R/p.T);
k620 = p.CIII.k260*exp(dGo/p.R/p.T);
k460 = p.CIII.k640*exp(dGi/p.R/p.T);

% State Transition Rates

% Coupled QH2 oxidation /c3 reduction
kfQH2c3_12 = p.CIII.k120*QH2/p.CIII.KQH2o./P_Qo.*c3/p.CIII.Kc3./P_c.*PISP;
krQH2c3_12 = k210.*c2/p.CIII.Kc2./P_c.*PISP./PQo;
kfQH2c3_34 = p.CIII.k340*QH2/p.CIII.KQH2o./P_Qo.*c3/p.CIII.Kc3./P_c.*PISP;
krQH2c3_34 = k430.*c2/p.CIII.Kc2./P_c.*PISP./PQo;
kfQH2c3_45 = p.CIII.k450*QH2/p.CIII.KQH2o./P_Qo.*c3/p.CIII.Kc3./P_c.*PISP.*rQo./PQo;
krQH2c3_45 = k540.*c2/p.CIII.Kc2./P_c.*PISP;
kfQH2c3_26 = p.CIII.k260*QH2/p.CIII.KQH2o./P_Qo.*c3/p.CIII.Kc3./P_c.*PISP.*rQo./PQo;
krQH2c3_26 = k620.*c2/p.CIII.Kc2./P_c.*PISP;

% Q uptake at SQ site
kfQ_23 = p.CIII.k230*rQo./PQo.*rQi2./PQi2.*exp(-p.CIII.beta1*p.F*dPsi/p.R/p.T/2);
krQ_23 = k320./PQi.*exp(p.CIII.beta1*p.F*dPsi/p.R/p.T/2);
kfQ_64 = p.CIII.k640*rQi2./PQi2.*exp(-p.CIII.beta1*p.F*dPsi/p.R/p.T/2);
krQ_64 = k460./PQo./PQi.*exp(p.CIII.beta1*p.F*dPsi/p.R/p.T/2);

% QH2 regeneration
kfQH2_41 = p.CIII.k410.*rQo./PQo.*rQi./PQi.*exp(-p.CIII.beta1*p.F*dPsi/p.R/p.T/2);
krQH2_41 = k140./PQi2.*exp(p.CIII.beta1*p.F*dPsi/p.R/p.T/2);
kfQH2_52 = p.CIII.k520*rQi./PQi.*exp(-p.CIII.beta1*p.F*dPsi/p.R/p.T/2);
krQH2_52 = k250./PQo./PQi2.*exp(p.CIII.beta1*p.F*dPsi/p.R/p.T/2);

% SO production
kfSO_21 = p.CIII.kSO.*O2./PQo;
krSO_21 = kSOr.*SO.*Q/p.CIII.KQo./P_Qo;
kfSO_43 = p.CIII.kSO.*O2./PQo;
krSO_43 = kSOr.*SO.*Q/p.CIII.KQo./P_Qo;
kfSO_54 = p.CIII.kSO.*O2;
krSO_54 = kSOr.*SO.*rQo./PQo.*Q/p.CIII.KQo./P_Qo;
kfSO_62 = p.CIII.kSO.*O2;
krSO_62 = kSOr.*SO.*rQo./PQo.*Q/p.CIII.KQo./P_Qo;

% QH2o + c3 -> SQo + c2 + 2H+
k12 = kfQH2c3_12 + krSO_21;
k21 = krQH2c3_12 + kfSO_21;

% bL -> bH
k23 = kfQ_23;
k32 = krQ_23; 

% QH2o + c3 -> SQo + c2 + 2H+
k34 = kfQH2c3_34 + krSO_43;
k43 = krQH2c3_34 + kfSO_43;

% bL -> bH
k41 = kfQH2_41;
k14 = krQH2_41;

% QH2o + c3 -> SQo + c2 + 2H+
k45 = kfQH2c3_45 + krSO_54;
k54 = krQH2c3_45 + kfSO_54;

% bL -> bH
k52 = kfQH2_52;
k25 = krQH2_52; 

% QH2o + c3 -> SQo + c2 + 2H+
k26 = kfQH2c3_26 + krSO_62;
k62 = krQH2c3_26 + kfSO_62;

% bL -> bH
k64 = kfQ_64;
k46 = krQ_64; 

% Steady-State Fractional Occupancies (solved analytically)
E1=(k21.*k32.*k41.*k52.*k62 + k21.*k32.*k41.*k52.*k64 + k21.*k32.*k41.*k54.*k62 + k21.*k32.*k43.*k52.*k62 + k21.*k34.*k41.*k52.*k62 + k21.*k32.*k41.*k54.*k64 + k21.*k32.*k43.*k52.*k64 + k21.*k32.*k43.*k54.*k62 + k21.*k32.*k45.*k52.*k62 + k21.*k34.*k41.*k52.*k64 + k21.*k34.*k41.*k54.*k62 + k23.*k34.*k41.*k52.*k62 + k21.*k32.*k46.*k52.*k62 + k21.*k32.*k43.*k54.*k64 + k21.*k32.*k45.*k52.*k64 + k21.*k34.*k41.*k54.*k64 + k21.*k34.*k45.*k52.*k62 + k23.*k34.*k41.*k52.*k64 + k23.*k34.*k41.*k54.*k62 + k25.*k32.*k41.*k54.*k62 + k21.*k32.*k46.*k54.*k62 + k21.*k34.*k46.*k52.*k62 + k26.*k32.*k41.*k52.*k64 + k21.*k34.*k45.*k52.*k64 + k23.*k34.*k41.*k54.*k64 + k25.*k32.*k41.*k54.*k64 + k25.*k34.*k41.*k54.*k62 + k21.*k34.*k46.*k54.*k62 + k26.*k32.*k41.*k54.*k64 + k26.*k34.*k41.*k52.*k64 + k25.*k34.*k41.*k54.*k64 + k26.*k34.*k41.*k54.*k64)./(k12.*k26.*k32.*k41.*k52 + k12.*k26.*k32.*k41.*k54 + k12.*k26.*k32.*k43.*k52 + k12.*k26.*k34.*k41.*k52 + k14.*k21.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k52 + k12.*k26.*k32.*k43.*k54 + k12.*k26.*k32.*k45.*k52 + k12.*k26.*k34.*k41.*k54 + k14.*k21.*k32.*k46.*k54 + k14.*k21.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k52 + k12.*k26.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k54 + k12.*k25.*k32.*k46.*k54 + k12.*k26.*k34.*k45.*k52 + k14.*k21.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k54 + k14.*k26.*k32.*k45.*k52 + k12.*k26.*k32.*k46.*k54 + k12.*k26.*k34.*k46.*k52 + k14.*k26.*k32.*k46.*k52 + k12.*k25.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k54 + k14.*k25.*k32.*k46.*k54 + k14.*k26.*k34.*k45.*k52 + k12.*k25.*k32.*k41.*k62 + k12.*k26.*k34.*k46.*k54 + k14.*k26.*k32.*k46.*k54 + k14.*k26.*k34.*k46.*k52 + k14.*k25.*k34.*k46.*k54 + k12.*k25.*k32.*k41.*k64 + k12.*k25.*k32.*k43.*k62 + k12.*k25.*k34.*k41.*k62 + k14.*k21.*k32.*k45.*k62 + k14.*k26.*k34.*k46.*k54 + k12.*k23.*k34.*k45.*k62 + k12.*k25.*k32.*k43.*k64 + k12.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k41.*k64 + k14.*k21.*k32.*k45.*k64 + k14.*k21.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k62 + k12.*k25.*k32.*k46.*k62 + k12.*k23.*k34.*k45.*k64 + k12.*k25.*k32.*k45.*k64 + k12.*k25.*k34.*k45.*k62 + k14.*k21.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k64 + k14.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k46.*k62 + k12.*k26.*k32.*k45.*k64 + k14.*k25.*k32.*k46.*k62 + k12.*k25.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k64 + k14.*k25.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k62 + k12.*k26.*k34.*k45.*k64 + k14.*k21.*k32.*k52.*k62 + k14.*k25.*k34.*k46.*k62 + k14.*k26.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k64 + k12.*k23.*k34.*k52.*k62 + k14.*k21.*k32.*k52.*k64 + k14.*k21.*k32.*k54.*k62 + k14.*k21.*k34.*k52.*k62 + k14.*k26.*k34.*k45.*k64 + k12.*k23.*k34.*k52.*k64 + k12.*k23.*k34.*k54.*k62 + k12.*k25.*k32.*k54.*k62 + k14.*k21.*k32.*k54.*k64 + k14.*k21.*k34.*k52.*k64 + k14.*k21.*k34.*k54.*k62 + k14.*k23.*k34.*k52.*k62 + k12.*k26.*k32.*k52.*k64 + k12.*k23.*k34.*k54.*k64 + k12.*k25.*k32.*k54.*k64 + k12.*k25.*k34.*k54.*k62 + k14.*k21.*k34.*k54.*k64 + k14.*k23.*k34.*k52.*k64 + k14.*k23.*k34.*k54.*k62 + k14.*k25.*k32.*k54.*k62 + k12.*k26.*k32.*k54.*k64 + k12.*k26.*k34.*k52.*k64 + k14.*k26.*k32.*k52.*k64 + k12.*k25.*k34.*k54.*k64 + k14.*k23.*k34.*k54.*k64 + k14.*k25.*k32.*k54.*k64 + k14.*k25.*k34.*k54.*k62 + k12.*k23.*k41.*k52.*k62 + k12.*k26.*k34.*k54.*k64 + k14.*k26.*k32.*k54.*k64 + k14.*k26.*k34.*k52.*k64 + k14.*k25.*k34.*k54.*k64 + k12.*k23.*k41.*k52.*k64 + k12.*k23.*k41.*k54.*k62 + k12.*k23.*k43.*k52.*k62 + k14.*k21.*k43.*k52.*k62 + k14.*k26.*k34.*k54.*k64 + k12.*k23.*k41.*k54.*k64 + k12.*k23.*k43.*k52.*k64 + k12.*k23.*k43.*k54.*k62 + k12.*k23.*k45.*k52.*k62 + k14.*k21.*k43.*k52.*k64 + k14.*k21.*k43.*k54.*k62 + k14.*k23.*k43.*k52.*k62 + k12.*k23.*k46.*k52.*k62 + k12.*k23.*k43.*k54.*k64 + k12.*k23.*k45.*k52.*k64 + k12.*k25.*k43.*k54.*k62 + k14.*k21.*k43.*k54.*k64 + k14.*k23.*k43.*k52.*k64 + k14.*k23.*k43.*k54.*k62 + k14.*k23.*k45.*k52.*k62 + k12.*k23.*k46.*k54.*k62 + k12.*k26.*k43.*k52.*k64 + k14.*k23.*k46.*k52.*k62 + k12.*k25.*k43.*k54.*k64 + k14.*k23.*k43.*k54.*k64 + k14.*k23.*k45.*k52.*k64 + k14.*k25.*k43.*k54.*k62 + k12.*k26.*k43.*k54.*k64 + k12.*k32.*k41.*k52.*k62 + k14.*k23.*k46.*k54.*k62 + k14.*k26.*k43.*k52.*k64 + k14.*k25.*k43.*k54.*k64 + k12.*k32.*k41.*k52.*k64 + k12.*k32.*k41.*k54.*k62 + k12.*k32.*k43.*k52.*k62 + k12.*k34.*k41.*k52.*k62 + k14.*k26.*k43.*k54.*k64 + k12.*k32.*k41.*k54.*k64 + k12.*k32.*k43.*k52.*k64 + k12.*k32.*k43.*k54.*k62 + k12.*k32.*k45.*k52.*k62 + k12.*k34.*k41.*k52.*k64 + k12.*k34.*k41.*k54.*k62 + k14.*k32.*k43.*k52.*k62 + k12.*k32.*k46.*k52.*k62 + k12.*k32.*k43.*k54.*k64 + k12.*k32.*k45.*k52.*k64 + k12.*k34.*k41.*k54.*k64 + k12.*k34.*k45.*k52.*k62 + k14.*k32.*k43.*k52.*k64 + k14.*k32.*k43.*k54.*k62 + k14.*k32.*k45.*k52.*k62 + k12.*k32.*k46.*k54.*k62 + k12.*k34.*k46.*k52.*k62 + k14.*k32.*k46.*k52.*k62 + k12.*k34.*k45.*k52.*k64 + k14.*k32.*k43.*k54.*k64 + k14.*k32.*k45.*k52.*k64 + k14.*k34.*k45.*k52.*k62 + k12.*k34.*k46.*k54.*k62 + k14.*k32.*k46.*k54.*k62 + k14.*k34.*k46.*k52.*k62 + k21.*k32.*k41.*k52.*k62 + k14.*k34.*k45.*k52.*k64 + k14.*k34.*k46.*k54.*k62 + k21.*k32.*k41.*k52.*k64 + k21.*k32.*k41.*k54.*k62 + k21.*k32.*k43.*k52.*k62 + k21.*k34.*k41.*k52.*k62 + k21.*k32.*k41.*k54.*k64 + k21.*k32.*k43.*k52.*k64 + k21.*k32.*k43.*k54.*k62 + k21.*k32.*k45.*k52.*k62 + k21.*k34.*k41.*k52.*k64 + k21.*k34.*k41.*k54.*k62 + k23.*k34.*k41.*k52.*k62 + k21.*k32.*k46.*k52.*k62 + k21.*k32.*k43.*k54.*k64 + k21.*k32.*k45.*k52.*k64 + k21.*k34.*k41.*k54.*k64 + k21.*k34.*k45.*k52.*k62 + k23.*k34.*k41.*k52.*k64 + k23.*k34.*k41.*k54.*k62 + k25.*k32.*k41.*k54.*k62 + k21.*k32.*k46.*k54.*k62 + k21.*k34.*k46.*k52.*k62 + k26.*k32.*k41.*k52.*k64 + k21.*k34.*k45.*k52.*k64 + k23.*k34.*k41.*k54.*k64 + k25.*k32.*k41.*k54.*k64 + k25.*k34.*k41.*k54.*k62 + k21.*k34.*k46.*k54.*k62 + k26.*k32.*k41.*k54.*k64 + k26.*k34.*k41.*k52.*k64 + k25.*k34.*k41.*k54.*k64 + k26.*k34.*k41.*k54.*k64);
E2=(k12.*k32.*k41.*k52.*k62 + k12.*k32.*k41.*k52.*k64 + k12.*k32.*k41.*k54.*k62 + k12.*k32.*k43.*k52.*k62 + k12.*k34.*k41.*k52.*k62 + k12.*k32.*k41.*k54.*k64 + k12.*k32.*k43.*k52.*k64 + k12.*k32.*k43.*k54.*k62 + k12.*k32.*k45.*k52.*k62 + k12.*k34.*k41.*k52.*k64 + k12.*k34.*k41.*k54.*k62 + k14.*k32.*k43.*k52.*k62 + k12.*k32.*k46.*k52.*k62 + k12.*k32.*k43.*k54.*k64 + k12.*k32.*k45.*k52.*k64 + k12.*k34.*k41.*k54.*k64 + k12.*k34.*k45.*k52.*k62 + k14.*k32.*k43.*k52.*k64 + k14.*k32.*k43.*k54.*k62 + k14.*k32.*k45.*k52.*k62 + k12.*k32.*k46.*k54.*k62 + k12.*k34.*k46.*k52.*k62 + k14.*k32.*k46.*k52.*k62 + k12.*k34.*k45.*k52.*k64 + k14.*k32.*k43.*k54.*k64 + k14.*k32.*k45.*k52.*k64 + k14.*k34.*k45.*k52.*k62 + k12.*k34.*k46.*k54.*k62 + k14.*k32.*k46.*k54.*k62 + k14.*k34.*k46.*k52.*k62 + k14.*k34.*k45.*k52.*k64 + k14.*k34.*k46.*k54.*k62)./(k12.*k26.*k32.*k41.*k52 + k12.*k26.*k32.*k41.*k54 + k12.*k26.*k32.*k43.*k52 + k12.*k26.*k34.*k41.*k52 + k14.*k21.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k52 + k12.*k26.*k32.*k43.*k54 + k12.*k26.*k32.*k45.*k52 + k12.*k26.*k34.*k41.*k54 + k14.*k21.*k32.*k46.*k54 + k14.*k21.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k52 + k12.*k26.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k54 + k12.*k25.*k32.*k46.*k54 + k12.*k26.*k34.*k45.*k52 + k14.*k21.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k54 + k14.*k26.*k32.*k45.*k52 + k12.*k26.*k32.*k46.*k54 + k12.*k26.*k34.*k46.*k52 + k14.*k26.*k32.*k46.*k52 + k12.*k25.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k54 + k14.*k25.*k32.*k46.*k54 + k14.*k26.*k34.*k45.*k52 + k12.*k25.*k32.*k41.*k62 + k12.*k26.*k34.*k46.*k54 + k14.*k26.*k32.*k46.*k54 + k14.*k26.*k34.*k46.*k52 + k14.*k25.*k34.*k46.*k54 + k12.*k25.*k32.*k41.*k64 + k12.*k25.*k32.*k43.*k62 + k12.*k25.*k34.*k41.*k62 + k14.*k21.*k32.*k45.*k62 + k14.*k26.*k34.*k46.*k54 + k12.*k23.*k34.*k45.*k62 + k12.*k25.*k32.*k43.*k64 + k12.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k41.*k64 + k14.*k21.*k32.*k45.*k64 + k14.*k21.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k62 + k12.*k25.*k32.*k46.*k62 + k12.*k23.*k34.*k45.*k64 + k12.*k25.*k32.*k45.*k64 + k12.*k25.*k34.*k45.*k62 + k14.*k21.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k64 + k14.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k46.*k62 + k12.*k26.*k32.*k45.*k64 + k14.*k25.*k32.*k46.*k62 + k12.*k25.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k64 + k14.*k25.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k62 + k12.*k26.*k34.*k45.*k64 + k14.*k21.*k32.*k52.*k62 + k14.*k25.*k34.*k46.*k62 + k14.*k26.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k64 + k12.*k23.*k34.*k52.*k62 + k14.*k21.*k32.*k52.*k64 + k14.*k21.*k32.*k54.*k62 + k14.*k21.*k34.*k52.*k62 + k14.*k26.*k34.*k45.*k64 + k12.*k23.*k34.*k52.*k64 + k12.*k23.*k34.*k54.*k62 + k12.*k25.*k32.*k54.*k62 + k14.*k21.*k32.*k54.*k64 + k14.*k21.*k34.*k52.*k64 + k14.*k21.*k34.*k54.*k62 + k14.*k23.*k34.*k52.*k62 + k12.*k26.*k32.*k52.*k64 + k12.*k23.*k34.*k54.*k64 + k12.*k25.*k32.*k54.*k64 + k12.*k25.*k34.*k54.*k62 + k14.*k21.*k34.*k54.*k64 + k14.*k23.*k34.*k52.*k64 + k14.*k23.*k34.*k54.*k62 + k14.*k25.*k32.*k54.*k62 + k12.*k26.*k32.*k54.*k64 + k12.*k26.*k34.*k52.*k64 + k14.*k26.*k32.*k52.*k64 + k12.*k25.*k34.*k54.*k64 + k14.*k23.*k34.*k54.*k64 + k14.*k25.*k32.*k54.*k64 + k14.*k25.*k34.*k54.*k62 + k12.*k23.*k41.*k52.*k62 + k12.*k26.*k34.*k54.*k64 + k14.*k26.*k32.*k54.*k64 + k14.*k26.*k34.*k52.*k64 + k14.*k25.*k34.*k54.*k64 + k12.*k23.*k41.*k52.*k64 + k12.*k23.*k41.*k54.*k62 + k12.*k23.*k43.*k52.*k62 + k14.*k21.*k43.*k52.*k62 + k14.*k26.*k34.*k54.*k64 + k12.*k23.*k41.*k54.*k64 + k12.*k23.*k43.*k52.*k64 + k12.*k23.*k43.*k54.*k62 + k12.*k23.*k45.*k52.*k62 + k14.*k21.*k43.*k52.*k64 + k14.*k21.*k43.*k54.*k62 + k14.*k23.*k43.*k52.*k62 + k12.*k23.*k46.*k52.*k62 + k12.*k23.*k43.*k54.*k64 + k12.*k23.*k45.*k52.*k64 + k12.*k25.*k43.*k54.*k62 + k14.*k21.*k43.*k54.*k64 + k14.*k23.*k43.*k52.*k64 + k14.*k23.*k43.*k54.*k62 + k14.*k23.*k45.*k52.*k62 + k12.*k23.*k46.*k54.*k62 + k12.*k26.*k43.*k52.*k64 + k14.*k23.*k46.*k52.*k62 + k12.*k25.*k43.*k54.*k64 + k14.*k23.*k43.*k54.*k64 + k14.*k23.*k45.*k52.*k64 + k14.*k25.*k43.*k54.*k62 + k12.*k26.*k43.*k54.*k64 + k12.*k32.*k41.*k52.*k62 + k14.*k23.*k46.*k54.*k62 + k14.*k26.*k43.*k52.*k64 + k14.*k25.*k43.*k54.*k64 + k12.*k32.*k41.*k52.*k64 + k12.*k32.*k41.*k54.*k62 + k12.*k32.*k43.*k52.*k62 + k12.*k34.*k41.*k52.*k62 + k14.*k26.*k43.*k54.*k64 + k12.*k32.*k41.*k54.*k64 + k12.*k32.*k43.*k52.*k64 + k12.*k32.*k43.*k54.*k62 + k12.*k32.*k45.*k52.*k62 + k12.*k34.*k41.*k52.*k64 + k12.*k34.*k41.*k54.*k62 + k14.*k32.*k43.*k52.*k62 + k12.*k32.*k46.*k52.*k62 + k12.*k32.*k43.*k54.*k64 + k12.*k32.*k45.*k52.*k64 + k12.*k34.*k41.*k54.*k64 + k12.*k34.*k45.*k52.*k62 + k14.*k32.*k43.*k52.*k64 + k14.*k32.*k43.*k54.*k62 + k14.*k32.*k45.*k52.*k62 + k12.*k32.*k46.*k54.*k62 + k12.*k34.*k46.*k52.*k62 + k14.*k32.*k46.*k52.*k62 + k12.*k34.*k45.*k52.*k64 + k14.*k32.*k43.*k54.*k64 + k14.*k32.*k45.*k52.*k64 + k14.*k34.*k45.*k52.*k62 + k12.*k34.*k46.*k54.*k62 + k14.*k32.*k46.*k54.*k62 + k14.*k34.*k46.*k52.*k62 + k21.*k32.*k41.*k52.*k62 + k14.*k34.*k45.*k52.*k64 + k14.*k34.*k46.*k54.*k62 + k21.*k32.*k41.*k52.*k64 + k21.*k32.*k41.*k54.*k62 + k21.*k32.*k43.*k52.*k62 + k21.*k34.*k41.*k52.*k62 + k21.*k32.*k41.*k54.*k64 + k21.*k32.*k43.*k52.*k64 + k21.*k32.*k43.*k54.*k62 + k21.*k32.*k45.*k52.*k62 + k21.*k34.*k41.*k52.*k64 + k21.*k34.*k41.*k54.*k62 + k23.*k34.*k41.*k52.*k62 + k21.*k32.*k46.*k52.*k62 + k21.*k32.*k43.*k54.*k64 + k21.*k32.*k45.*k52.*k64 + k21.*k34.*k41.*k54.*k64 + k21.*k34.*k45.*k52.*k62 + k23.*k34.*k41.*k52.*k64 + k23.*k34.*k41.*k54.*k62 + k25.*k32.*k41.*k54.*k62 + k21.*k32.*k46.*k54.*k62 + k21.*k34.*k46.*k52.*k62 + k26.*k32.*k41.*k52.*k64 + k21.*k34.*k45.*k52.*k64 + k23.*k34.*k41.*k54.*k64 + k25.*k32.*k41.*k54.*k64 + k25.*k34.*k41.*k54.*k62 + k21.*k34.*k46.*k54.*k62 + k26.*k32.*k41.*k54.*k64 + k26.*k34.*k41.*k52.*k64 + k25.*k34.*k41.*k54.*k64 + k26.*k34.*k41.*k54.*k64);
E3=(k12.*k23.*k41.*k52.*k62 + k12.*k23.*k41.*k52.*k64 + k12.*k23.*k41.*k54.*k62 + k12.*k23.*k43.*k52.*k62 + k14.*k21.*k43.*k52.*k62 + k12.*k23.*k41.*k54.*k64 + k12.*k23.*k43.*k52.*k64 + k12.*k23.*k43.*k54.*k62 + k12.*k23.*k45.*k52.*k62 + k14.*k21.*k43.*k52.*k64 + k14.*k21.*k43.*k54.*k62 + k14.*k23.*k43.*k52.*k62 + k12.*k23.*k46.*k52.*k62 + k12.*k23.*k43.*k54.*k64 + k12.*k23.*k45.*k52.*k64 + k12.*k25.*k43.*k54.*k62 + k14.*k21.*k43.*k54.*k64 + k14.*k23.*k43.*k52.*k64 + k14.*k23.*k43.*k54.*k62 + k14.*k23.*k45.*k52.*k62 + k12.*k23.*k46.*k54.*k62 + k12.*k26.*k43.*k52.*k64 + k14.*k23.*k46.*k52.*k62 + k12.*k25.*k43.*k54.*k64 + k14.*k23.*k43.*k54.*k64 + k14.*k23.*k45.*k52.*k64 + k14.*k25.*k43.*k54.*k62 + k12.*k26.*k43.*k54.*k64 + k14.*k23.*k46.*k54.*k62 + k14.*k26.*k43.*k52.*k64 + k14.*k25.*k43.*k54.*k64 + k14.*k26.*k43.*k54.*k64)./(k12.*k26.*k32.*k41.*k52 + k12.*k26.*k32.*k41.*k54 + k12.*k26.*k32.*k43.*k52 + k12.*k26.*k34.*k41.*k52 + k14.*k21.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k52 + k12.*k26.*k32.*k43.*k54 + k12.*k26.*k32.*k45.*k52 + k12.*k26.*k34.*k41.*k54 + k14.*k21.*k32.*k46.*k54 + k14.*k21.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k52 + k12.*k26.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k54 + k12.*k25.*k32.*k46.*k54 + k12.*k26.*k34.*k45.*k52 + k14.*k21.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k54 + k14.*k26.*k32.*k45.*k52 + k12.*k26.*k32.*k46.*k54 + k12.*k26.*k34.*k46.*k52 + k14.*k26.*k32.*k46.*k52 + k12.*k25.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k54 + k14.*k25.*k32.*k46.*k54 + k14.*k26.*k34.*k45.*k52 + k12.*k25.*k32.*k41.*k62 + k12.*k26.*k34.*k46.*k54 + k14.*k26.*k32.*k46.*k54 + k14.*k26.*k34.*k46.*k52 + k14.*k25.*k34.*k46.*k54 + k12.*k25.*k32.*k41.*k64 + k12.*k25.*k32.*k43.*k62 + k12.*k25.*k34.*k41.*k62 + k14.*k21.*k32.*k45.*k62 + k14.*k26.*k34.*k46.*k54 + k12.*k23.*k34.*k45.*k62 + k12.*k25.*k32.*k43.*k64 + k12.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k41.*k64 + k14.*k21.*k32.*k45.*k64 + k14.*k21.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k62 + k12.*k25.*k32.*k46.*k62 + k12.*k23.*k34.*k45.*k64 + k12.*k25.*k32.*k45.*k64 + k12.*k25.*k34.*k45.*k62 + k14.*k21.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k64 + k14.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k46.*k62 + k12.*k26.*k32.*k45.*k64 + k14.*k25.*k32.*k46.*k62 + k12.*k25.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k64 + k14.*k25.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k62 + k12.*k26.*k34.*k45.*k64 + k14.*k21.*k32.*k52.*k62 + k14.*k25.*k34.*k46.*k62 + k14.*k26.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k64 + k12.*k23.*k34.*k52.*k62 + k14.*k21.*k32.*k52.*k64 + k14.*k21.*k32.*k54.*k62 + k14.*k21.*k34.*k52.*k62 + k14.*k26.*k34.*k45.*k64 + k12.*k23.*k34.*k52.*k64 + k12.*k23.*k34.*k54.*k62 + k12.*k25.*k32.*k54.*k62 + k14.*k21.*k32.*k54.*k64 + k14.*k21.*k34.*k52.*k64 + k14.*k21.*k34.*k54.*k62 + k14.*k23.*k34.*k52.*k62 + k12.*k26.*k32.*k52.*k64 + k12.*k23.*k34.*k54.*k64 + k12.*k25.*k32.*k54.*k64 + k12.*k25.*k34.*k54.*k62 + k14.*k21.*k34.*k54.*k64 + k14.*k23.*k34.*k52.*k64 + k14.*k23.*k34.*k54.*k62 + k14.*k25.*k32.*k54.*k62 + k12.*k26.*k32.*k54.*k64 + k12.*k26.*k34.*k52.*k64 + k14.*k26.*k32.*k52.*k64 + k12.*k25.*k34.*k54.*k64 + k14.*k23.*k34.*k54.*k64 + k14.*k25.*k32.*k54.*k64 + k14.*k25.*k34.*k54.*k62 + k12.*k23.*k41.*k52.*k62 + k12.*k26.*k34.*k54.*k64 + k14.*k26.*k32.*k54.*k64 + k14.*k26.*k34.*k52.*k64 + k14.*k25.*k34.*k54.*k64 + k12.*k23.*k41.*k52.*k64 + k12.*k23.*k41.*k54.*k62 + k12.*k23.*k43.*k52.*k62 + k14.*k21.*k43.*k52.*k62 + k14.*k26.*k34.*k54.*k64 + k12.*k23.*k41.*k54.*k64 + k12.*k23.*k43.*k52.*k64 + k12.*k23.*k43.*k54.*k62 + k12.*k23.*k45.*k52.*k62 + k14.*k21.*k43.*k52.*k64 + k14.*k21.*k43.*k54.*k62 + k14.*k23.*k43.*k52.*k62 + k12.*k23.*k46.*k52.*k62 + k12.*k23.*k43.*k54.*k64 + k12.*k23.*k45.*k52.*k64 + k12.*k25.*k43.*k54.*k62 + k14.*k21.*k43.*k54.*k64 + k14.*k23.*k43.*k52.*k64 + k14.*k23.*k43.*k54.*k62 + k14.*k23.*k45.*k52.*k62 + k12.*k23.*k46.*k54.*k62 + k12.*k26.*k43.*k52.*k64 + k14.*k23.*k46.*k52.*k62 + k12.*k25.*k43.*k54.*k64 + k14.*k23.*k43.*k54.*k64 + k14.*k23.*k45.*k52.*k64 + k14.*k25.*k43.*k54.*k62 + k12.*k26.*k43.*k54.*k64 + k12.*k32.*k41.*k52.*k62 + k14.*k23.*k46.*k54.*k62 + k14.*k26.*k43.*k52.*k64 + k14.*k25.*k43.*k54.*k64 + k12.*k32.*k41.*k52.*k64 + k12.*k32.*k41.*k54.*k62 + k12.*k32.*k43.*k52.*k62 + k12.*k34.*k41.*k52.*k62 + k14.*k26.*k43.*k54.*k64 + k12.*k32.*k41.*k54.*k64 + k12.*k32.*k43.*k52.*k64 + k12.*k32.*k43.*k54.*k62 + k12.*k32.*k45.*k52.*k62 + k12.*k34.*k41.*k52.*k64 + k12.*k34.*k41.*k54.*k62 + k14.*k32.*k43.*k52.*k62 + k12.*k32.*k46.*k52.*k62 + k12.*k32.*k43.*k54.*k64 + k12.*k32.*k45.*k52.*k64 + k12.*k34.*k41.*k54.*k64 + k12.*k34.*k45.*k52.*k62 + k14.*k32.*k43.*k52.*k64 + k14.*k32.*k43.*k54.*k62 + k14.*k32.*k45.*k52.*k62 + k12.*k32.*k46.*k54.*k62 + k12.*k34.*k46.*k52.*k62 + k14.*k32.*k46.*k52.*k62 + k12.*k34.*k45.*k52.*k64 + k14.*k32.*k43.*k54.*k64 + k14.*k32.*k45.*k52.*k64 + k14.*k34.*k45.*k52.*k62 + k12.*k34.*k46.*k54.*k62 + k14.*k32.*k46.*k54.*k62 + k14.*k34.*k46.*k52.*k62 + k21.*k32.*k41.*k52.*k62 + k14.*k34.*k45.*k52.*k64 + k14.*k34.*k46.*k54.*k62 + k21.*k32.*k41.*k52.*k64 + k21.*k32.*k41.*k54.*k62 + k21.*k32.*k43.*k52.*k62 + k21.*k34.*k41.*k52.*k62 + k21.*k32.*k41.*k54.*k64 + k21.*k32.*k43.*k52.*k64 + k21.*k32.*k43.*k54.*k62 + k21.*k32.*k45.*k52.*k62 + k21.*k34.*k41.*k52.*k64 + k21.*k34.*k41.*k54.*k62 + k23.*k34.*k41.*k52.*k62 + k21.*k32.*k46.*k52.*k62 + k21.*k32.*k43.*k54.*k64 + k21.*k32.*k45.*k52.*k64 + k21.*k34.*k41.*k54.*k64 + k21.*k34.*k45.*k52.*k62 + k23.*k34.*k41.*k52.*k64 + k23.*k34.*k41.*k54.*k62 + k25.*k32.*k41.*k54.*k62 + k21.*k32.*k46.*k54.*k62 + k21.*k34.*k46.*k52.*k62 + k26.*k32.*k41.*k52.*k64 + k21.*k34.*k45.*k52.*k64 + k23.*k34.*k41.*k54.*k64 + k25.*k32.*k41.*k54.*k64 + k25.*k34.*k41.*k54.*k62 + k21.*k34.*k46.*k54.*k62 + k26.*k32.*k41.*k54.*k64 + k26.*k34.*k41.*k52.*k64 + k25.*k34.*k41.*k54.*k64 + k26.*k34.*k41.*k54.*k64);
E4=(k14.*k21.*k32.*k52.*k62 + k12.*k23.*k34.*k52.*k62 + k14.*k21.*k32.*k52.*k64 + k14.*k21.*k32.*k54.*k62 + k14.*k21.*k34.*k52.*k62 + k12.*k23.*k34.*k52.*k64 + k12.*k23.*k34.*k54.*k62 + k12.*k25.*k32.*k54.*k62 + k14.*k21.*k32.*k54.*k64 + k14.*k21.*k34.*k52.*k64 + k14.*k21.*k34.*k54.*k62 + k14.*k23.*k34.*k52.*k62 + k12.*k26.*k32.*k52.*k64 + k12.*k23.*k34.*k54.*k64 + k12.*k25.*k32.*k54.*k64 + k12.*k25.*k34.*k54.*k62 + k14.*k21.*k34.*k54.*k64 + k14.*k23.*k34.*k52.*k64 + k14.*k23.*k34.*k54.*k62 + k14.*k25.*k32.*k54.*k62 + k12.*k26.*k32.*k54.*k64 + k12.*k26.*k34.*k52.*k64 + k14.*k26.*k32.*k52.*k64 + k12.*k25.*k34.*k54.*k64 + k14.*k23.*k34.*k54.*k64 + k14.*k25.*k32.*k54.*k64 + k14.*k25.*k34.*k54.*k62 + k12.*k26.*k34.*k54.*k64 + k14.*k26.*k32.*k54.*k64 + k14.*k26.*k34.*k52.*k64 + k14.*k25.*k34.*k54.*k64 + k14.*k26.*k34.*k54.*k64)./(k12.*k26.*k32.*k41.*k52 + k12.*k26.*k32.*k41.*k54 + k12.*k26.*k32.*k43.*k52 + k12.*k26.*k34.*k41.*k52 + k14.*k21.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k52 + k12.*k26.*k32.*k43.*k54 + k12.*k26.*k32.*k45.*k52 + k12.*k26.*k34.*k41.*k54 + k14.*k21.*k32.*k46.*k54 + k14.*k21.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k52 + k12.*k26.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k54 + k12.*k25.*k32.*k46.*k54 + k12.*k26.*k34.*k45.*k52 + k14.*k21.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k54 + k14.*k26.*k32.*k45.*k52 + k12.*k26.*k32.*k46.*k54 + k12.*k26.*k34.*k46.*k52 + k14.*k26.*k32.*k46.*k52 + k12.*k25.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k54 + k14.*k25.*k32.*k46.*k54 + k14.*k26.*k34.*k45.*k52 + k12.*k25.*k32.*k41.*k62 + k12.*k26.*k34.*k46.*k54 + k14.*k26.*k32.*k46.*k54 + k14.*k26.*k34.*k46.*k52 + k14.*k25.*k34.*k46.*k54 + k12.*k25.*k32.*k41.*k64 + k12.*k25.*k32.*k43.*k62 + k12.*k25.*k34.*k41.*k62 + k14.*k21.*k32.*k45.*k62 + k14.*k26.*k34.*k46.*k54 + k12.*k23.*k34.*k45.*k62 + k12.*k25.*k32.*k43.*k64 + k12.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k41.*k64 + k14.*k21.*k32.*k45.*k64 + k14.*k21.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k62 + k12.*k25.*k32.*k46.*k62 + k12.*k23.*k34.*k45.*k64 + k12.*k25.*k32.*k45.*k64 + k12.*k25.*k34.*k45.*k62 + k14.*k21.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k64 + k14.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k46.*k62 + k12.*k26.*k32.*k45.*k64 + k14.*k25.*k32.*k46.*k62 + k12.*k25.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k64 + k14.*k25.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k62 + k12.*k26.*k34.*k45.*k64 + k14.*k21.*k32.*k52.*k62 + k14.*k25.*k34.*k46.*k62 + k14.*k26.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k64 + k12.*k23.*k34.*k52.*k62 + k14.*k21.*k32.*k52.*k64 + k14.*k21.*k32.*k54.*k62 + k14.*k21.*k34.*k52.*k62 + k14.*k26.*k34.*k45.*k64 + k12.*k23.*k34.*k52.*k64 + k12.*k23.*k34.*k54.*k62 + k12.*k25.*k32.*k54.*k62 + k14.*k21.*k32.*k54.*k64 + k14.*k21.*k34.*k52.*k64 + k14.*k21.*k34.*k54.*k62 + k14.*k23.*k34.*k52.*k62 + k12.*k26.*k32.*k52.*k64 + k12.*k23.*k34.*k54.*k64 + k12.*k25.*k32.*k54.*k64 + k12.*k25.*k34.*k54.*k62 + k14.*k21.*k34.*k54.*k64 + k14.*k23.*k34.*k52.*k64 + k14.*k23.*k34.*k54.*k62 + k14.*k25.*k32.*k54.*k62 + k12.*k26.*k32.*k54.*k64 + k12.*k26.*k34.*k52.*k64 + k14.*k26.*k32.*k52.*k64 + k12.*k25.*k34.*k54.*k64 + k14.*k23.*k34.*k54.*k64 + k14.*k25.*k32.*k54.*k64 + k14.*k25.*k34.*k54.*k62 + k12.*k23.*k41.*k52.*k62 + k12.*k26.*k34.*k54.*k64 + k14.*k26.*k32.*k54.*k64 + k14.*k26.*k34.*k52.*k64 + k14.*k25.*k34.*k54.*k64 + k12.*k23.*k41.*k52.*k64 + k12.*k23.*k41.*k54.*k62 + k12.*k23.*k43.*k52.*k62 + k14.*k21.*k43.*k52.*k62 + k14.*k26.*k34.*k54.*k64 + k12.*k23.*k41.*k54.*k64 + k12.*k23.*k43.*k52.*k64 + k12.*k23.*k43.*k54.*k62 + k12.*k23.*k45.*k52.*k62 + k14.*k21.*k43.*k52.*k64 + k14.*k21.*k43.*k54.*k62 + k14.*k23.*k43.*k52.*k62 + k12.*k23.*k46.*k52.*k62 + k12.*k23.*k43.*k54.*k64 + k12.*k23.*k45.*k52.*k64 + k12.*k25.*k43.*k54.*k62 + k14.*k21.*k43.*k54.*k64 + k14.*k23.*k43.*k52.*k64 + k14.*k23.*k43.*k54.*k62 + k14.*k23.*k45.*k52.*k62 + k12.*k23.*k46.*k54.*k62 + k12.*k26.*k43.*k52.*k64 + k14.*k23.*k46.*k52.*k62 + k12.*k25.*k43.*k54.*k64 + k14.*k23.*k43.*k54.*k64 + k14.*k23.*k45.*k52.*k64 + k14.*k25.*k43.*k54.*k62 + k12.*k26.*k43.*k54.*k64 + k12.*k32.*k41.*k52.*k62 + k14.*k23.*k46.*k54.*k62 + k14.*k26.*k43.*k52.*k64 + k14.*k25.*k43.*k54.*k64 + k12.*k32.*k41.*k52.*k64 + k12.*k32.*k41.*k54.*k62 + k12.*k32.*k43.*k52.*k62 + k12.*k34.*k41.*k52.*k62 + k14.*k26.*k43.*k54.*k64 + k12.*k32.*k41.*k54.*k64 + k12.*k32.*k43.*k52.*k64 + k12.*k32.*k43.*k54.*k62 + k12.*k32.*k45.*k52.*k62 + k12.*k34.*k41.*k52.*k64 + k12.*k34.*k41.*k54.*k62 + k14.*k32.*k43.*k52.*k62 + k12.*k32.*k46.*k52.*k62 + k12.*k32.*k43.*k54.*k64 + k12.*k32.*k45.*k52.*k64 + k12.*k34.*k41.*k54.*k64 + k12.*k34.*k45.*k52.*k62 + k14.*k32.*k43.*k52.*k64 + k14.*k32.*k43.*k54.*k62 + k14.*k32.*k45.*k52.*k62 + k12.*k32.*k46.*k54.*k62 + k12.*k34.*k46.*k52.*k62 + k14.*k32.*k46.*k52.*k62 + k12.*k34.*k45.*k52.*k64 + k14.*k32.*k43.*k54.*k64 + k14.*k32.*k45.*k52.*k64 + k14.*k34.*k45.*k52.*k62 + k12.*k34.*k46.*k54.*k62 + k14.*k32.*k46.*k54.*k62 + k14.*k34.*k46.*k52.*k62 + k21.*k32.*k41.*k52.*k62 + k14.*k34.*k45.*k52.*k64 + k14.*k34.*k46.*k54.*k62 + k21.*k32.*k41.*k52.*k64 + k21.*k32.*k41.*k54.*k62 + k21.*k32.*k43.*k52.*k62 + k21.*k34.*k41.*k52.*k62 + k21.*k32.*k41.*k54.*k64 + k21.*k32.*k43.*k52.*k64 + k21.*k32.*k43.*k54.*k62 + k21.*k32.*k45.*k52.*k62 + k21.*k34.*k41.*k52.*k64 + k21.*k34.*k41.*k54.*k62 + k23.*k34.*k41.*k52.*k62 + k21.*k32.*k46.*k52.*k62 + k21.*k32.*k43.*k54.*k64 + k21.*k32.*k45.*k52.*k64 + k21.*k34.*k41.*k54.*k64 + k21.*k34.*k45.*k52.*k62 + k23.*k34.*k41.*k52.*k64 + k23.*k34.*k41.*k54.*k62 + k25.*k32.*k41.*k54.*k62 + k21.*k32.*k46.*k54.*k62 + k21.*k34.*k46.*k52.*k62 + k26.*k32.*k41.*k52.*k64 + k21.*k34.*k45.*k52.*k64 + k23.*k34.*k41.*k54.*k64 + k25.*k32.*k41.*k54.*k64 + k25.*k34.*k41.*k54.*k62 + k21.*k34.*k46.*k54.*k62 + k26.*k32.*k41.*k54.*k64 + k26.*k34.*k41.*k52.*k64 + k25.*k34.*k41.*k54.*k64 + k26.*k34.*k41.*k54.*k64);
E5=(k12.*k25.*k32.*k41.*k62 + k12.*k25.*k32.*k41.*k64 + k12.*k25.*k32.*k43.*k62 + k12.*k25.*k34.*k41.*k62 + k14.*k21.*k32.*k45.*k62 + k12.*k23.*k34.*k45.*k62 + k12.*k25.*k32.*k43.*k64 + k12.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k41.*k64 + k14.*k21.*k32.*k45.*k64 + k14.*k21.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k62 + k12.*k25.*k32.*k46.*k62 + k12.*k23.*k34.*k45.*k64 + k12.*k25.*k32.*k45.*k64 + k12.*k25.*k34.*k45.*k62 + k14.*k21.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k64 + k14.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k46.*k62 + k12.*k26.*k32.*k45.*k64 + k14.*k25.*k32.*k46.*k62 + k12.*k25.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k64 + k14.*k25.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k62 + k12.*k26.*k34.*k45.*k64 + k14.*k25.*k34.*k46.*k62 + k14.*k26.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k64 + k14.*k26.*k34.*k45.*k64)./(k12.*k26.*k32.*k41.*k52 + k12.*k26.*k32.*k41.*k54 + k12.*k26.*k32.*k43.*k52 + k12.*k26.*k34.*k41.*k52 + k14.*k21.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k52 + k12.*k26.*k32.*k43.*k54 + k12.*k26.*k32.*k45.*k52 + k12.*k26.*k34.*k41.*k54 + k14.*k21.*k32.*k46.*k54 + k14.*k21.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k52 + k12.*k26.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k54 + k12.*k25.*k32.*k46.*k54 + k12.*k26.*k34.*k45.*k52 + k14.*k21.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k54 + k14.*k26.*k32.*k45.*k52 + k12.*k26.*k32.*k46.*k54 + k12.*k26.*k34.*k46.*k52 + k14.*k26.*k32.*k46.*k52 + k12.*k25.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k54 + k14.*k25.*k32.*k46.*k54 + k14.*k26.*k34.*k45.*k52 + k12.*k25.*k32.*k41.*k62 + k12.*k26.*k34.*k46.*k54 + k14.*k26.*k32.*k46.*k54 + k14.*k26.*k34.*k46.*k52 + k14.*k25.*k34.*k46.*k54 + k12.*k25.*k32.*k41.*k64 + k12.*k25.*k32.*k43.*k62 + k12.*k25.*k34.*k41.*k62 + k14.*k21.*k32.*k45.*k62 + k14.*k26.*k34.*k46.*k54 + k12.*k23.*k34.*k45.*k62 + k12.*k25.*k32.*k43.*k64 + k12.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k41.*k64 + k14.*k21.*k32.*k45.*k64 + k14.*k21.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k62 + k12.*k25.*k32.*k46.*k62 + k12.*k23.*k34.*k45.*k64 + k12.*k25.*k32.*k45.*k64 + k12.*k25.*k34.*k45.*k62 + k14.*k21.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k64 + k14.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k46.*k62 + k12.*k26.*k32.*k45.*k64 + k14.*k25.*k32.*k46.*k62 + k12.*k25.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k64 + k14.*k25.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k62 + k12.*k26.*k34.*k45.*k64 + k14.*k21.*k32.*k52.*k62 + k14.*k25.*k34.*k46.*k62 + k14.*k26.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k64 + k12.*k23.*k34.*k52.*k62 + k14.*k21.*k32.*k52.*k64 + k14.*k21.*k32.*k54.*k62 + k14.*k21.*k34.*k52.*k62 + k14.*k26.*k34.*k45.*k64 + k12.*k23.*k34.*k52.*k64 + k12.*k23.*k34.*k54.*k62 + k12.*k25.*k32.*k54.*k62 + k14.*k21.*k32.*k54.*k64 + k14.*k21.*k34.*k52.*k64 + k14.*k21.*k34.*k54.*k62 + k14.*k23.*k34.*k52.*k62 + k12.*k26.*k32.*k52.*k64 + k12.*k23.*k34.*k54.*k64 + k12.*k25.*k32.*k54.*k64 + k12.*k25.*k34.*k54.*k62 + k14.*k21.*k34.*k54.*k64 + k14.*k23.*k34.*k52.*k64 + k14.*k23.*k34.*k54.*k62 + k14.*k25.*k32.*k54.*k62 + k12.*k26.*k32.*k54.*k64 + k12.*k26.*k34.*k52.*k64 + k14.*k26.*k32.*k52.*k64 + k12.*k25.*k34.*k54.*k64 + k14.*k23.*k34.*k54.*k64 + k14.*k25.*k32.*k54.*k64 + k14.*k25.*k34.*k54.*k62 + k12.*k23.*k41.*k52.*k62 + k12.*k26.*k34.*k54.*k64 + k14.*k26.*k32.*k54.*k64 + k14.*k26.*k34.*k52.*k64 + k14.*k25.*k34.*k54.*k64 + k12.*k23.*k41.*k52.*k64 + k12.*k23.*k41.*k54.*k62 + k12.*k23.*k43.*k52.*k62 + k14.*k21.*k43.*k52.*k62 + k14.*k26.*k34.*k54.*k64 + k12.*k23.*k41.*k54.*k64 + k12.*k23.*k43.*k52.*k64 + k12.*k23.*k43.*k54.*k62 + k12.*k23.*k45.*k52.*k62 + k14.*k21.*k43.*k52.*k64 + k14.*k21.*k43.*k54.*k62 + k14.*k23.*k43.*k52.*k62 + k12.*k23.*k46.*k52.*k62 + k12.*k23.*k43.*k54.*k64 + k12.*k23.*k45.*k52.*k64 + k12.*k25.*k43.*k54.*k62 + k14.*k21.*k43.*k54.*k64 + k14.*k23.*k43.*k52.*k64 + k14.*k23.*k43.*k54.*k62 + k14.*k23.*k45.*k52.*k62 + k12.*k23.*k46.*k54.*k62 + k12.*k26.*k43.*k52.*k64 + k14.*k23.*k46.*k52.*k62 + k12.*k25.*k43.*k54.*k64 + k14.*k23.*k43.*k54.*k64 + k14.*k23.*k45.*k52.*k64 + k14.*k25.*k43.*k54.*k62 + k12.*k26.*k43.*k54.*k64 + k12.*k32.*k41.*k52.*k62 + k14.*k23.*k46.*k54.*k62 + k14.*k26.*k43.*k52.*k64 + k14.*k25.*k43.*k54.*k64 + k12.*k32.*k41.*k52.*k64 + k12.*k32.*k41.*k54.*k62 + k12.*k32.*k43.*k52.*k62 + k12.*k34.*k41.*k52.*k62 + k14.*k26.*k43.*k54.*k64 + k12.*k32.*k41.*k54.*k64 + k12.*k32.*k43.*k52.*k64 + k12.*k32.*k43.*k54.*k62 + k12.*k32.*k45.*k52.*k62 + k12.*k34.*k41.*k52.*k64 + k12.*k34.*k41.*k54.*k62 + k14.*k32.*k43.*k52.*k62 + k12.*k32.*k46.*k52.*k62 + k12.*k32.*k43.*k54.*k64 + k12.*k32.*k45.*k52.*k64 + k12.*k34.*k41.*k54.*k64 + k12.*k34.*k45.*k52.*k62 + k14.*k32.*k43.*k52.*k64 + k14.*k32.*k43.*k54.*k62 + k14.*k32.*k45.*k52.*k62 + k12.*k32.*k46.*k54.*k62 + k12.*k34.*k46.*k52.*k62 + k14.*k32.*k46.*k52.*k62 + k12.*k34.*k45.*k52.*k64 + k14.*k32.*k43.*k54.*k64 + k14.*k32.*k45.*k52.*k64 + k14.*k34.*k45.*k52.*k62 + k12.*k34.*k46.*k54.*k62 + k14.*k32.*k46.*k54.*k62 + k14.*k34.*k46.*k52.*k62 + k21.*k32.*k41.*k52.*k62 + k14.*k34.*k45.*k52.*k64 + k14.*k34.*k46.*k54.*k62 + k21.*k32.*k41.*k52.*k64 + k21.*k32.*k41.*k54.*k62 + k21.*k32.*k43.*k52.*k62 + k21.*k34.*k41.*k52.*k62 + k21.*k32.*k41.*k54.*k64 + k21.*k32.*k43.*k52.*k64 + k21.*k32.*k43.*k54.*k62 + k21.*k32.*k45.*k52.*k62 + k21.*k34.*k41.*k52.*k64 + k21.*k34.*k41.*k54.*k62 + k23.*k34.*k41.*k52.*k62 + k21.*k32.*k46.*k52.*k62 + k21.*k32.*k43.*k54.*k64 + k21.*k32.*k45.*k52.*k64 + k21.*k34.*k41.*k54.*k64 + k21.*k34.*k45.*k52.*k62 + k23.*k34.*k41.*k52.*k64 + k23.*k34.*k41.*k54.*k62 + k25.*k32.*k41.*k54.*k62 + k21.*k32.*k46.*k54.*k62 + k21.*k34.*k46.*k52.*k62 + k26.*k32.*k41.*k52.*k64 + k21.*k34.*k45.*k52.*k64 + k23.*k34.*k41.*k54.*k64 + k25.*k32.*k41.*k54.*k64 + k25.*k34.*k41.*k54.*k62 + k21.*k34.*k46.*k54.*k62 + k26.*k32.*k41.*k54.*k64 + k26.*k34.*k41.*k52.*k64 + k25.*k34.*k41.*k54.*k64 + k26.*k34.*k41.*k54.*k64);
% E6=(k12.*k26.*k32.*k41.*k52 + k12.*k26.*k32.*k41.*k54 + k12.*k26.*k32.*k43.*k52 + k12.*k26.*k34.*k41.*k52 + k14.*k21.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k52 + k12.*k26.*k32.*k43.*k54 + k12.*k26.*k32.*k45.*k52 + k12.*k26.*k34.*k41.*k54 + k14.*k21.*k32.*k46.*k54 + k14.*k21.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k52 + k12.*k26.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k54 + k12.*k25.*k32.*k46.*k54 + k12.*k26.*k34.*k45.*k52 + k14.*k21.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k54 + k14.*k26.*k32.*k45.*k52 + k12.*k26.*k32.*k46.*k54 + k12.*k26.*k34.*k46.*k52 + k14.*k26.*k32.*k46.*k52 + k12.*k25.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k54 + k14.*k25.*k32.*k46.*k54 + k14.*k26.*k34.*k45.*k52 + k12.*k26.*k34.*k46.*k54 + k14.*k26.*k32.*k46.*k54 + k14.*k26.*k34.*k46.*k52 + k14.*k25.*k34.*k46.*k54 + k14.*k26.*k34.*k46.*k54)./(k12.*k26.*k32.*k41.*k52 + k12.*k26.*k32.*k41.*k54 + k12.*k26.*k32.*k43.*k52 + k12.*k26.*k34.*k41.*k52 + k14.*k21.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k52 + k12.*k26.*k32.*k43.*k54 + k12.*k26.*k32.*k45.*k52 + k12.*k26.*k34.*k41.*k54 + k14.*k21.*k32.*k46.*k54 + k14.*k21.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k52 + k12.*k26.*k32.*k46.*k52 + k12.*k23.*k34.*k46.*k54 + k12.*k25.*k32.*k46.*k54 + k12.*k26.*k34.*k45.*k52 + k14.*k21.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k52 + k14.*k26.*k32.*k43.*k54 + k14.*k26.*k32.*k45.*k52 + k12.*k26.*k32.*k46.*k54 + k12.*k26.*k34.*k46.*k52 + k14.*k26.*k32.*k46.*k52 + k12.*k25.*k34.*k46.*k54 + k14.*k23.*k34.*k46.*k54 + k14.*k25.*k32.*k46.*k54 + k14.*k26.*k34.*k45.*k52 + k12.*k25.*k32.*k41.*k62 + k12.*k26.*k34.*k46.*k54 + k14.*k26.*k32.*k46.*k54 + k14.*k26.*k34.*k46.*k52 + k14.*k25.*k34.*k46.*k54 + k12.*k25.*k32.*k41.*k64 + k12.*k25.*k32.*k43.*k62 + k12.*k25.*k34.*k41.*k62 + k14.*k21.*k32.*k45.*k62 + k14.*k26.*k34.*k46.*k54 + k12.*k23.*k34.*k45.*k62 + k12.*k25.*k32.*k43.*k64 + k12.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k41.*k64 + k14.*k21.*k32.*k45.*k64 + k14.*k21.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k62 + k12.*k25.*k32.*k46.*k62 + k12.*k23.*k34.*k45.*k64 + k12.*k25.*k32.*k45.*k64 + k12.*k25.*k34.*k45.*k62 + k14.*k21.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k62 + k14.*k25.*k32.*k43.*k64 + k14.*k25.*k32.*k45.*k62 + k12.*k25.*k34.*k46.*k62 + k12.*k26.*k32.*k45.*k64 + k14.*k25.*k32.*k46.*k62 + k12.*k25.*k34.*k45.*k64 + k14.*k23.*k34.*k45.*k64 + k14.*k25.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k62 + k12.*k26.*k34.*k45.*k64 + k14.*k21.*k32.*k52.*k62 + k14.*k25.*k34.*k46.*k62 + k14.*k26.*k32.*k45.*k64 + k14.*k25.*k34.*k45.*k64 + k12.*k23.*k34.*k52.*k62 + k14.*k21.*k32.*k52.*k64 + k14.*k21.*k32.*k54.*k62 + k14.*k21.*k34.*k52.*k62 + k14.*k26.*k34.*k45.*k64 + k12.*k23.*k34.*k52.*k64 + k12.*k23.*k34.*k54.*k62 + k12.*k25.*k32.*k54.*k62 + k14.*k21.*k32.*k54.*k64 + k14.*k21.*k34.*k52.*k64 + k14.*k21.*k34.*k54.*k62 + k14.*k23.*k34.*k52.*k62 + k12.*k26.*k32.*k52.*k64 + k12.*k23.*k34.*k54.*k64 + k12.*k25.*k32.*k54.*k64 + k12.*k25.*k34.*k54.*k62 + k14.*k21.*k34.*k54.*k64 + k14.*k23.*k34.*k52.*k64 + k14.*k23.*k34.*k54.*k62 + k14.*k25.*k32.*k54.*k62 + k12.*k26.*k32.*k54.*k64 + k12.*k26.*k34.*k52.*k64 + k14.*k26.*k32.*k52.*k64 + k12.*k25.*k34.*k54.*k64 + k14.*k23.*k34.*k54.*k64 + k14.*k25.*k32.*k54.*k64 + k14.*k25.*k34.*k54.*k62 + k12.*k23.*k41.*k52.*k62 + k12.*k26.*k34.*k54.*k64 + k14.*k26.*k32.*k54.*k64 + k14.*k26.*k34.*k52.*k64 + k14.*k25.*k34.*k54.*k64 + k12.*k23.*k41.*k52.*k64 + k12.*k23.*k41.*k54.*k62 + k12.*k23.*k43.*k52.*k62 + k14.*k21.*k43.*k52.*k62 + k14.*k26.*k34.*k54.*k64 + k12.*k23.*k41.*k54.*k64 + k12.*k23.*k43.*k52.*k64 + k12.*k23.*k43.*k54.*k62 + k12.*k23.*k45.*k52.*k62 + k14.*k21.*k43.*k52.*k64 + k14.*k21.*k43.*k54.*k62 + k14.*k23.*k43.*k52.*k62 + k12.*k23.*k46.*k52.*k62 + k12.*k23.*k43.*k54.*k64 + k12.*k23.*k45.*k52.*k64 + k12.*k25.*k43.*k54.*k62 + k14.*k21.*k43.*k54.*k64 + k14.*k23.*k43.*k52.*k64 + k14.*k23.*k43.*k54.*k62 + k14.*k23.*k45.*k52.*k62 + k12.*k23.*k46.*k54.*k62 + k12.*k26.*k43.*k52.*k64 + k14.*k23.*k46.*k52.*k62 + k12.*k25.*k43.*k54.*k64 + k14.*k23.*k43.*k54.*k64 + k14.*k23.*k45.*k52.*k64 + k14.*k25.*k43.*k54.*k62 + k12.*k26.*k43.*k54.*k64 + k12.*k32.*k41.*k52.*k62 + k14.*k23.*k46.*k54.*k62 + k14.*k26.*k43.*k52.*k64 + k14.*k25.*k43.*k54.*k64 + k12.*k32.*k41.*k52.*k64 + k12.*k32.*k41.*k54.*k62 + k12.*k32.*k43.*k52.*k62 + k12.*k34.*k41.*k52.*k62 + k14.*k26.*k43.*k54.*k64 + k12.*k32.*k41.*k54.*k64 + k12.*k32.*k43.*k52.*k64 + k12.*k32.*k43.*k54.*k62 + k12.*k32.*k45.*k52.*k62 + k12.*k34.*k41.*k52.*k64 + k12.*k34.*k41.*k54.*k62 + k14.*k32.*k43.*k52.*k62 + k12.*k32.*k46.*k52.*k62 + k12.*k32.*k43.*k54.*k64 + k12.*k32.*k45.*k52.*k64 + k12.*k34.*k41.*k54.*k64 + k12.*k34.*k45.*k52.*k62 + k14.*k32.*k43.*k52.*k64 + k14.*k32.*k43.*k54.*k62 + k14.*k32.*k45.*k52.*k62 + k12.*k32.*k46.*k54.*k62 + k12.*k34.*k46.*k52.*k62 + k14.*k32.*k46.*k52.*k62 + k12.*k34.*k45.*k52.*k64 + k14.*k32.*k43.*k54.*k64 + k14.*k32.*k45.*k52.*k64 + k14.*k34.*k45.*k52.*k62 + k12.*k34.*k46.*k54.*k62 + k14.*k32.*k46.*k54.*k62 + k14.*k34.*k46.*k52.*k62 + k21.*k32.*k41.*k52.*k62 + k14.*k34.*k45.*k52.*k64 + k14.*k34.*k46.*k54.*k62 + k21.*k32.*k41.*k52.*k64 + k21.*k32.*k41.*k54.*k62 + k21.*k32.*k43.*k52.*k62 + k21.*k34.*k41.*k52.*k62 + k21.*k32.*k41.*k54.*k64 + k21.*k32.*k43.*k52.*k64 + k21.*k32.*k43.*k54.*k62 + k21.*k32.*k45.*k52.*k62 + k21.*k34.*k41.*k52.*k64 + k21.*k34.*k41.*k54.*k62 + k23.*k34.*k41.*k52.*k62 + k21.*k32.*k46.*k52.*k62 + k21.*k32.*k43.*k54.*k64 + k21.*k32.*k45.*k52.*k64 + k21.*k34.*k41.*k54.*k64 + k21.*k34.*k45.*k52.*k62 + k23.*k34.*k41.*k52.*k64 + k23.*k34.*k41.*k54.*k62 + k25.*k32.*k41.*k54.*k62 + k21.*k32.*k46.*k54.*k62 + k21.*k34.*k46.*k52.*k62 + k26.*k32.*k41.*k52.*k64 + k21.*k34.*k45.*k52.*k64 + k23.*k34.*k41.*k54.*k64 + k25.*k32.*k41.*k54.*k64 + k25.*k34.*k41.*k54.*k62 + k21.*k34.*k46.*k54.*k62 + k26.*k32.*k41.*k54.*k64 + k26.*k34.*k41.*k52.*k64 + k25.*k34.*k41.*k54.*k64 + k26.*k34.*k41.*k54.*k64);
E6 = 1 - E1 - E2 - E3 - E4 - E5;

% Net Turnover Rates
% Jc3 = p.CIII.x*Pi_activation.*((kfQH2c3_12.*E1 - krQH2c3_12.*E2) + (kfQH2c3_34.*E3 - krQH2c3_34.*E4) + (kfQH2c3_45.*E4 - krQH2c3_45.*E5) + (kfQH2c3_26.*E2 - krQH2c3_26.*E6)); % cyt c3+ production
JQH2 = p.CIII.x.*((kfQH2c3_12.*E1 - krQH2c3_12.*E2) + (kfQH2c3_34.*E3 - krQH2c3_34.*E4) + (kfQH2c3_45.*E4 - krQH2c3_45.*E5) + (kfQH2c3_26.*E2 - krQH2c3_26.*E6) - (kfQH2_41.*E4 - krQH2_41.*E1) - (kfQH2_52.*E5 - krQH2_52.*E2)); % QH2 consumption
% JQ = p.CIII.x*Pi_activation.*((kfQH2c3_12.*E1 - krQH2c3_12.*E2) + (kfQH2c3_34.*E3 - krQH2c3_34.*E4) + (kfQH2c3_45.*E4 - krQH2c3_45.*E5) + (kfQH2c3_26.*E2 - krQH2c3_26.*E6) - (kfQ_23.*E2 - krQ_23.*E3) - (kfQ_64.*E6 - krQ_64.*E4)); % Q production, identical to QH2 oxidation
JSO = p.CIII.x.*((kfSO_21.*E2 - krSO_21.*E1) + (kfSO_43.*E4 - krSO_43.*E3) + (kfSO_54.*E5 - krSO_54.*E4) + (kfSO_62.*E6 - krSO_62.*E2)); % superoxide production
% JHp = p.CIII.x*2*Pi_activation.*((kfQH2c3_12.*E1 - krQH2c3_12.*E2) + (kfQH2c3_34.*E3 - krQH2c3_34.*E4) + (kfQH2c3_45.*E4 - krQH2c3_45.*E5) + (kfQH2c3_26.*E2 - krQH2c3_26.*E6)); % scalar protons released on IMS side
% JHn = p.CIII.x*2*Pi_activation.*((kfQH2_41.*E4 - krQH2_41.*E1) + (kfQH2_52.*E5 - krQH2_52.*E2)); % scalar protons taken up from matrix (also, 1 charge equiv is moved across membrane for every 2H+)
% JdPsi = p.CIII.x*Pi_activation.*(1*((kfQ_23.*E2 - krQ_23.*E3) + (kfQ_64.*E6 - krQ_64.*E4)) + 1*((kfQH2_41.*E4 - krQH2_41.*E1) + (kfQH2_52.*E5 - krQH2_52.*E2))); % charge movement across IMM

% for numerical stability, enforce strict stoichiometric coupling
Jc3 = 2*JQH2 - JSO;
JQ = JQH2;
JHp = 4*JQH2 - 2*JSO;
JHn = 2*(JQH2 - JSO);
JdPsi = 2*(JQH2 - JSO);

% Output
varargout = {Jc3, JQH2, JQ, JSO, JHp, JHn, JdPsi};
end

%% Complex IV
% based on Murphy and Brand 1987
function J = ETC4(x,p)

% Define SVs into generalized notation
c2 = x(p.i.c2_i,:);         % M
c3 = x(p.i.c3_i,:);         % M
O2 = x(p.i.O2_x,:);       % M
Hx = x(p.i.H_x,:);          % M
Hi = x(p.i.H_c,:);          % M
dPsi = x(p.i.dPsi,:);       % mV

%Calculate pH independnt dG0
dG0 = 2*p.dGf(p.if.c3) + p.dGf(p.if.H2O) ...
    - 2*p.dGf(p.if.c2) - 0.5*p.dGf(p.if.O2);

AC4=dG0+4*p.F*dPsi;  % partial dG spanning complex, 4H+/2e-
Keq=exp(-AC4./p.R./p.T).*(Hx.^4./Hi.^2);

KM = p.CIV.KM; % M
beta = p.CIV.beta; % unitless

J = p.CIV.x.*(O2./(O2+p.CIV.k_O2)).*(c2.^2./(c2.^2 + KM^2*(1+beta*exp(dPsi*2*p.F/p.R/p.T)))).*(1 - (c3.^2./c2.^2./O2.^.5)./Keq);
end

%% Intrinsic Proton Leak
function J = HLEAK(x,p)

% Define SVs into generalized notation
Hx = x(p.i.H_x,:);    % M
Hi = x(p.i.H_c,:);    % M
DPsi = x(p.i.dPsi,:);   % mV

x_HLE = p.mHleak.x;

J = x_HLE.*(Hi.*exp(p.F.*DPsi./p.R./p.T/2) ...
    -Hx.*exp(-p.F.*DPsi./p.R./p.T/2));
end

%% F1FO ATP Synthase
function J = F1FOATPASE(x,p)

% Define SVs into generalized notation
ADP = x(p.i.ADP_x,:);   % M
Pi = x(p.i.Pi_x,:);     % M
ATP = x(p.i.ATP_x,:);   % M
Hx = x(p.i.H_x,:);      % M
Hi = x(p.i.H_c,:);      % M
dPsi = x(p.i.dPsi,:);   % mV

% define reference reacation dG
dGr = p.dGf(p.if.ATP) + p.dGf(p.if.H2O) - p.dGf(p.if.ADP) ...
    - p.dGf(p.if.Pi); % kJ/mol

% if p.T==310.15
%     dGr = -5.643; % taken directly from BISEN (see notes notes for details)
% end

% define total free energy adding contribution to proton motive force
Keq0 = exp(-(dGr - (p.F1FO.nH)*p.F*dPsi)./p.R./p.T);

% define overall equlibrium constant
Keq = Keq0.*(Hi.^3./Hx.^2).*BP(p.if.ATP,'mtx',x,p) ...
    ./BP(p.if.ADP,'mtx',x,p)./BP(p.if.Pi,'mtx',x,p);

k1_F1FO = p.F1FO.x;

J = k1_F1FO*(Keq.*ADP.*Pi-ATP);
end

%% Metelkin ANT model
function J = ANT(x,p)

% Define SVs into generalized notation
ADP_i1 = x(p.i.ADP_c,:)./BP(p.if.ADP,'cyt',x,p); % ADP^3-
ATP_i1 = x(p.i.ATP_c,:)./BP(p.if.ATP,'cyt',x,p); % ATP^4-
ADP_x1 = x(p.i.ADP_x,:)./BP(p.if.ADP,'mtx',x,p); % ADP^3-
ATP_x1 = x(p.i.ATP_x,:)./BP(p.if.ATP,'mtx',x,p); % ATP^4-
DPsi = x(p.i.dPsi,:);   % mV

x_ANT = p.ANT.x;

del_D = 0.0167;
del_T = 0.0699;
k2_ANT = 9.54/60; % = 1.59e-1
k3_ANT = 30.05/60; % = 5.01e-1
K_D_o_ANT = 38.89e-6; 
K_T_o_ANT = 56.05e-6;
A = +0.2829;
B = -0.2086;
C = +0.2372;
fi = p.F*DPsi/p.R/p.T;
k2_ANT_fi = k2_ANT*exp((A*(-3)+B*(-4)+C)*fi);
k3_ANT_fi = k3_ANT*exp((A*(-4)+B*(-3)+C)*fi);

K_D_o_ANT_fi = K_D_o_ANT*exp(3*del_D*fi);
K_T_o_ANT_fi = K_T_o_ANT*exp(4*del_T*fi);
q = k3_ANT_fi.*K_D_o_ANT_fi.*exp(fi)./(k2_ANT_fi.*K_T_o_ANT_fi);
term2 = k2_ANT_fi.*ATP_x1.*ADP_i1.*q./K_D_o_ANT_fi ;
term3 = k3_ANT_fi.*ADP_x1.*ATP_i1./K_T_o_ANT_fi;
num = term2 - term3;
den = (1 + ATP_i1./K_T_o_ANT_fi + ADP_i1./K_D_o_ANT_fi) ...
    .*(ADP_x1 + ATP_x1.*q);

J = x_ANT*num./den; 
end

%% Inorganic Phosphate Carrier
function J = PIH(x,p)

% Define SVs into generalized notation
Pii = x(p.i.Pi_c,:);
Pix = x(p.i.Pi_x,:);
Hi = x(p.i.H_c,:);
Hx = x(p.i.H_x,:);

A = Pii.*Hi./p.KD(p.if.Pi,p.if.H)./BP(p.if.Pi,'cyt',x,p);
P = Pix.*Hx./p.KD(p.if.Pi,p.if.H)./BP(p.if.Pi,'mtx',x,p);

x_PIH = p.PIC.x;
k_PIH = p.PIC.kPi;

J = x_PIH/k_PIH*(Hi.*A - Hx.*P)./(1+A/k_PIH)./(1+P/k_PIH);
end

%% Potassium Hydrogen Exchange
function J = KHE(x,p)

k1_KHE = p.mKHE.x;
J = k1_KHE*(x(p.i.K_x,:).*x(p.i.H_c,:) - x(p.i.K_c,:).*x(p.i.H_x,:));
end

%% Basic ATPase function used to fit Vinnakota 2015 data

function J = ATPasem0(x,p)

ATP = x(p.i.ATP_c,:);   % M
ADP = x(p.i.ADP_c,:);   % M
Pi = x(p.i.Pi_c,:);     % M
H = x(p.i.H_c,:);       % M

dGr_atpase = p.dGf(p.if.ADP) + p.dGf(p.if.Pi) - p.dGf(p.if.ATP) ...
    - p.dGf(p.if.H2O);
Keq_atpase = exp(-dGr_atpase/p.R./p.T);
Keq = Keq_atpase.*(1./H) ...
    .*(BP(p.if.ADP,'cyt',x,p).*BP(p.if.Pi,'cyt',x,p)) ...
    ./BP(p.if.ATP,'cyt',x,p);

J = p.ATPase.x*p.ATPase.Etot./(1+x(p.i.ADP_c,:)/p.ATPase.Ki_ADP).*(1 - ADP.*Pi./ATP./Keq);
end

%% ATPase function used in Wu et al. 2008 J Physiol

function J = ATPasem1(x,p)

ATP = x(p.i.ATP_c,:);   % M
ADP = x(p.i.ADP_c,:);   % M
Pi = x(p.i.Pi_c,:);     % M

J = p.ATPase.Etot./(1+p.ATPase.R*Pi.*ADP./ATP);
end

%% Creatine Kinase 
% DGr0 taken from Wu et al. 2008 J Physiol
function J = CKm0(x,p)

ATP = x(p.i.ATP_c,:);   % M
ADP = x(p.i.ADP_c,:);   % M
PCr = x(p.i.PCr_c,:); % M
Cr = x(p.i.Cr_c,:); % M
H = x(p.i.H_c,:); % M

DGro =-52.57; % Wu et al. 2008 PNAS value
Keq = exp(-DGro/p.R/p.T)*BP(p.if.ATP,'cyt',x,p)./BP(p.if.ADP,'cyt',x,p).*H; % add BPs for Cr and PCr when pKs set
J = p.CK.x*( Keq.*ADP.*PCr -  ATP.*Cr);
end

%% Adenylate Kinase
function J = AKm0(x,p)

% Define SVs into generalized notation
ATP = x(p.i.ATP_c,:);   % M
ADP = x(p.i.ADP_c,:);   % M
AMP = x(p.i.AMP_c,:);   % M

dGr_AKo = p.dGf(p.if.ATP) + p.dGf(p.if.AMP) - 2*p.dGf(p.if.ADP);
Keq_AK = exp(-dGr_AKo/p.R./p.T);
Keq = Keq_AK.*BP(p.if.ATP,'cyt',x,p).*BP(p.if.AMP,'cyt',x,p) ...
    ./(BP(p.if.ADP,'cyt',x,p).*BP(p.if.ADP,'cyt',x,p));

J = p.AK.x*( Keq.*ADP.*ADP - AMP.*ATP );
end