%% Simulate FCCP Titration and Plot Results
clear

% load model parameters
load params


%% Initiale Model Parameters and ICs
xi0 = Model_ICs(p); % general model ICs

% define Temp
p.T = 310.15; % 37C
p.I = 0.17; % M
p = dGpKcorr(p); % adjust free energies and pKs for ionic strength and temp
p.exp = 0; % clamp all cytosolic variables

%% Set water/mito volume fraction (based on mito load)
p.Vol.H2O_cyt = 1e6; % (l cyto / l mito) % set high

%% Initialize Integrator Options

% Mass matrix
M=eye(p.i.length);
M(p.i.ATP_x,:)=0;M(p.i.NADH_x,:)=0;M(p.i.UQH2_x,:)=0;M(p.i.c2_i,:)=0;

% define integration absolute tolerances for matrix, ims and cytosol SVs
AbsTol_vec=1e-9.*ones(1,p.i.length);

% set global integrator options
options = odeset('RelTol',1e-4,'AbsTol',AbsTol_vec,'Vectorized','on', ...
    'MassSingular','yes','Mass',M,'MStateDependence','none','BDF','on');
% Note: cannot use non-negativity constraint with system of DAEs
p.max_cputime = 60; % s

%% Setup FCCP Simulations

xi0(p.i.H_c) = 10^-7.1;
xi0(p.i.Mg_c) = 1e-3;
xi0(p.i.ATP_c) = 1e-9;
xi0(p.i.ADP_c) = 1e-9;
xi0(p.i.K_c) = 0.13;
xi0(p.i.K_x) = 0.06;
xi0(p.i.Pi_c) = 1.2e-3;
xi0(p.i.O2_x) = 200e-6;

%%%%%%%%%%%%%%%%%%%%% Simulate Baseline %%%%%%%%%%%%%%%%%%%%
p.FCCP = 0;
p.ATPase.Etot = 0; % assume no work
[t0,x0] = ode15s(@DAEs,[0 600],xi0,options,p);
[f0, j0] = DAEs(t0',x0.',p);

% set ICs for 1 mM Pi simulations =clc 
xi1=x0(end,:);

clear t x f j
%%%%%%%%%%%%%%%% Simulate Ischemia/Anoxia %%%%%%%%%%%%%%%%%%%%
FCCP=logspace(-6,-1,81);
for i=1:numel(FCCP)
    p.FCCP=FCCP(i);
    [t{i},x{i}] = ode15s(@DAEs,[0 600],xi1,options,p);
    [f{i}, j{i}] = DAEs(t{i}',x{i}.',p);
end

clear QH2 dPsi JO2 c2 NADH r JDH JFCCP dpH
for i=1:numel(FCCP)
    QH2(i)=x{i}(end,p.i.UQH2_x);
    dPsi(i)=x{i}(end,p.i.dPsi);
    JO2(i)=j{i}.CIV(end);
    c2(i)=x{i}(end,p.i.c2_i);
    NADH(i)=x{i}(end,p.i.NADH_x);
    r(i)=x{i}(end,p.i.ADP_x).*x{i}(end,p.i.Pi_x)./x{i}(end,p.i.ATP_x);
    JDH(i)=j{i}.MitoDH(end);
    JFCCP(i)=j{i}.FCCP(end);
    dpH(i)=p.R*p.T/p.F*log(x{i}(end,p.i.H_c)./x{i}(end,p.i.H_x));
end

EhN = -320 - p.R*p.T/p.F/2*log(NADH./(p.Ntot-NADH));
EhQ = 60 - p.R*p.T/p.F/2*log(QH2./(p.Qtot-QH2));
Ehc = 245 - p.R*p.T/p.F*log(c2./(p.Ctot-c2));

%% Plot Figures

h1=figure;
a1=axes('parent',h1,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(log10(FCCP./dPsi),EhQ,'linewidth',4)
plot(log10(FCCP./dPsi),Ehc,'linewidth',4)
plot([0 0],[0 0],'linewidth',0.01)
plot(log10(FCCP./dPsi),dPsi,'linewidth',4)
xlabel('log_{10}[FCCP Activity] (mol H^+/s/l_{mito}/mV)','fontsize',32)
ylabel('Potential (mV)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a1,'ylim',[0 300],'ytick',[0 50 100 150 200 250 300],'xlim',[-8 -2])

h3=figure;
a3=axes('parent',h3,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(dPsi,QH2/p.Qtot,'linewidth',3)
plot(dPsi,c2/p.Ctot,'linewidth',3)
plot(dPsi,NADH/p.Ntot,'linewidth',3)
xlabel('\Delta\Psi (mV)','fontsize',32)
ylabel('fraction reduced','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a3,'xlim',[0 200])

O2conv = p.Vol.rho_m*60*(0.06/0.337)/2*1e9; % converts mol O/s/l mito to nmol O2/min/U CS

h4=figure;
a4=axes('parent',h4,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(dPsi,JO2*O2conv,'linewidth',3)
xlabel('\Delta\Psi (mV)','fontsize',32)
ylabel('J_{O{_2}} (nmolO_2/mg/min)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a4,'xlim',[0 200],'ylim',[0 300])