%% Simulate Ischemia/Reperfusion and Plot Results
clear
close all

% load model parameters
load params

% load data
load data

p.IRsimulation = true;

%% Initiale Model Parameters and ICs
xi0 = Model_ICs(p); % general model ICs

% define Temp
p.T = 310.15; % 37C
p.I = 0.17; % M
p = dGpKcorr(p); % adjust free energies and pKs for ionic strength and temp
p.exp = 1; % allow cytosolic SVs to change

%% Set water/mito volume fraction
p.Vol.H2O_cyt = 3; % (l cyto / l mito) mito approx. 30% V/V for cardiomyocytes

%% Initialize Integrator Options

% Mass matrix
M=eye(p.i.length);
M(p.i.ATP_x,:)=0;M(p.i.NADH_x,:)=0;M(p.i.UQH2_x,:)=0;M(p.i.c2_i,:)=0;M(p.i.PCr_c,:)=0;

% define integration absolute tolerances for matrix, ims and cytosol SVs
AbsTol_vec=1e-6.*ones(1,p.i.length);

% set global integrator options
options = odeset('RelTol',1e-3,'AbsTol',AbsTol_vec,'Vectorized','on', ...
    'MassSingular','yes','Mass',M,'MStateDependence','none','BDF','on');
% Note: cannot use non-negativity constraint with system of DAEs
p.max_cputime = 60; % s

% baseline values
bATPase = 2e-3; % set to yield intermediate work rate (JO2)
bSDH = 6e-4; % changes steady state JCII:JCI flux
bKeq = 1;

% altered values for I/R simulation 2
xSDH = 10;
xKeq = 10;

O2conv = p.Vol.rho_m*60*(0.06/0.337)/2*1e9; % converts mol O/s/l mito to nmol O2/min/U CS
%% Run I/R Simulation Protocol with no changes to model before or during
% Set ICs
xi0(p.i.H_c) = 10^-7.1;
xi0(p.i.Mg_c) = 1e-3;
xi0(p.i.ATP_c) = 10e-3;
xi0(p.i.ADP_c) = 1e-9;
xi0(p.i.AMP_c) = 1e-9;
xi0(p.i.K_c) = 0.13;
xi0(p.i.K_x) = 0.05;
xi0(p.i.Pi_c) = 1e-3;
xi0(p.i.Cr_c) = 10e-3;
xi0(p.i.PCr_c) = 20e-3;
xi0(p.i.O2_x) = 30e-6;

% set CII parameters
p.CII.x = bSDH;
p.CII.Keq = bKeq;

%%%%%%%%%%%%%%%%%%%%% Simulate Baseline %%%%%%%%%%%%%%%%%%%%

p.ATPase.Etot = bATPase; % M/s (baseline so JO2 at intermediate value)
[t0,x0] = ode15s(@DAEs,[0 60],xi0,options,p);
[f0, j0] = DAEs(t0',x0.',p);

% store end point for next simulation
xi1=x0(end,:);

%%%%%%%%%%%%%%%% Simulate Ischemia/Anoxia %%%%%%%%%%%%%%%%%%%%

xi1(p.i.O2_x) = 1e-9; % lower O2

[t1,x1] = ode15s(@DAEs,[0 60],xi1,options,p); % note, this should be 3 min
[f1, j1] = DAEs(t1',x1.',p);

xi2=x1(end,:);

%%%%%%%%%%%%%% Simulate Reperfusion %%%%%%%%%%%%%%%%%%%%%%%

xi2(p.i.O2_x) = 30e-6; % reset O2

[t2,x2] = ode15s(@DAEs,[0 60],xi2,options,p);
[f2, j2] = DAEs(t2',x2.',p);

xi3=x2(end,:);

%%%%%%%%%%%%%% Simulate Return to Baseline %%%%%%%%%%%%%%%

xi3(p.i.O2_x) = 30e-6;

[t3,x3] = ode15s(@DAEs,[0 60],xi3,options,p); % note, this should be 3 min
[f3, j3] = DAEs(t3',x3.',p);

% Compile Simulation Results for I/R Simulation 1
T0=[t0;t1+t0(end);t2+t1(end)+t0(end);t3+t2(end)+t1(end)+t0(end)];
X0=[x0;x1;x2;x3];
[F0,J0]=DAEs(T0.',X0.',p);

%% Run I/R Simulation Protocol with changes to model before or during
% Set ICs
xi0(p.i.H_c) = 10^-7.1;
xi0(p.i.Mg_c) = 1e-3;
xi0(p.i.ATP_c) = 10e-3;
xi0(p.i.ADP_c) = 1e-9;
xi0(p.i.AMP_c) = 1e-9;
xi0(p.i.K_c) = 0.13;
xi0(p.i.K_x) = 0.05;
xi0(p.i.Pi_c) = 1e-3;
xi0(p.i.Cr_c) = 10e-3;
xi0(p.i.PCr_c) = 20e-3;
xi0(p.i.O2_x) = 30e-6;

% set CII parameters
p.CII.x = bSDH;
p.CII.Keq = bKeq;

%%%%%%%%%%%%%%% Simulate Baseline %%%%%%%%%%%%%%%%%

p.ATPase.Etot = bATPase; % M/s
[t0,x0] = ode15s(@DAEs,[0 60],xi0,options,p);
[f0, j0] = DAEs(t0',x0.',p);

% set ICs for 1 mM Pi simulations =clc 
xi1=x0(end,:);

%%%%%%%%%%%%%%%% Simulate Ischemia/Anoxia %%%%%%%%%%%%%%%%%

xi1(p.i.O2_x) = 1e-9; % lower O2

[t1,x1] = ode15s(@DAEs,[0 60],xi1,options,p); % note, this should be 3 min
[f1, j1] = DAEs(t1',x1.',p);

xi2=x1(end,:);

%%%%%%%%%% Simulate Reperfusion %%%%%%%%%%%%%%%%%%%

xi2(p.i.O2_x) = 30e-6; % reset O2
p.CII.x = p.CII.x*xSDH;
p.CII.Keq = p.CII.Keq*xKeq;

[t2,x2] = ode15s(@DAEs,[0 60],xi2,options,p); % note, this should be 3 min
[f2, j2] = DAEs(t2',x2.',p);

xi3=x2(end,:);

%%%%%%%%%%%%%%% Simulate Return to Baseline %%%%%%%%%%%%%%

xi3(p.i.O2_x) = 30e-6;
p.CII.x = p.CII.x/xSDH; % reset
p.CII.Keq = p.CII.Keq/xKeq; % reset

[t3,x3] = ode15s(@DAEs,[0 60],xi3,options,p); % note, this should be 3 min
[f3, j3] = DAEs(t3',x3.',p);

% Compile Simulation Results for I/R Simulation Protocol 2
T1=[t0;t1+t0(end);t2+t1(end)+t0(end);t3+t2(end)+t1(end)+t0(end)];
X1=[x0;x1;x2;x3];
[F1,J1]=DAEs(T1.',X1.',p);

% correct for segment where CII.Keq was changed (f2 and j2)
in = numel(t1)+numel(t0)+1:numel(t2)+numel(t1)+numel(t0);
p.CII.x = 1e-2;
p.CII.Keq = 10;
[F1i,J1i]=DAEs(T1(in).',X1(in,:).',p);
F1(:,in)=F1i;
fieldnames1=fieldnames(J1i);
for i=1:numel(fieldnames1)
    try fieldnames2=eval(['fieldnames(J1i.' fieldnames1{i} ')']);
        for j=1:numel(fieldnames2)
            eval(['J1.' fieldnames1{i} '.' fieldnames2{j} '(in)=J1i.' fieldnames1{i} '.' fieldnames2{j} ';'])
        end
    catch
    eval(['J1.' fieldnames1{i} '(in)=J1i.' fieldnames1{i} ';'])
    end
end

%% Plot Figures

close all

yvar = 'UQH2';
h1=figure;
a1=axes('parent',h1,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(T0/60,X0(:,p.i.UQH2_x)/p.Qtot,'linewidth',4)
plot(T1/60,X1(:,p.i.UQH2_x)/p.Qtot,'linewidth',4)
xlabel('Time (min)','fontsize',32)
ylabel('fraction [UQH_2]_x','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a1,'ylim',[0 1])

yvar = 'dPsi';
h3=figure;
a3=axes('parent',h3,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(T0/60,X0(:,p.i.dPsi),'linewidth',4)
plot(T1/60,X1(:,p.i.dPsi),'linewidth',4)
xlabel('Time (min)','fontsize',32)
ylabel('\Delta\Psi (mV)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a3,'ylim',[120 190])

yvar = 'H2O2x';
h4=figure;
a4=axes('parent',h4,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(T0/60,X0(:,p.i.H2O2_x)*1e9,'linewidth',4)
plot(T1/60,X1(:,p.i.H2O2_x)*1e9,'linewidth',4)
xlabel('Time (min)','fontsize',32)
ylabel('[H_2O_2]_x (nM)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')

yvar = 'SOx';
h5=figure;
a5=axes('parent',h5,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(T0/60,X0(:,p.i.SO_x)*1e12,'linewidth',4)
plot(T1/60,X1(:,p.i.SO_x)*1e12,'linewidth',4)
xlabel('Time (min)','fontsize',32)
ylabel('[O_2^{.-}]_x (pM)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')

yvar = 'J_ROS';
h6=figure;
a6=axes('parent',h6,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(T0/60,J0.CIII.JSO/2*60*1e9*p.Vol.rho_m,'linewidth',4)
plot(T1/60,J1.CIII.JSO/2*60*1e9*p.Vol.rho_m,'linewidth',4)
plot(T0/60,(J0.CI.JSO/2+J0.CI.JH2O2)*60*1e9*p.Vol.rho_m,'--','color',[0 0.2 0.9],'linewidth',4)
plot(T1/60,(J1.CI.JSO/2+J1.CI.JH2O2)*60*1e9*p.Vol.rho_m,'--','color',[0.9 0.2 0],'linewidth',4)
xlabel('Time (min)','fontsize',32)
ylabel('J_{ROS} (nmol 2e^-/min/mg)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
% set(a6,'ylim',[0 0.3])

yvar = 'JO2';
h7=figure;
a7=axes('parent',h7,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(T0/60,J0.CIV*O2conv,'linewidth',4)
plot(T1/60,J1.CIV*O2conv,'linewidth',4)
xlabel('Time (min)','fontsize',32)
ylabel('J_{O_2} (nmol O_2/min/U CS)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a7,'ylim',[0 100],'ytick',[0 25 50 75 100])

yvar = 'JETC';
h8=figure;
a8=axes('parent',h8,'fontsize',28,'fontname','Times New Roman');
box on
hold on
% plot(T0/60,J0.CI.JNADH,T0/60,J0.CII,T0/60,J0.CIII.Jc3/2,T0/60,J0.CIV,'linewidth',4)
plot(T1/60,J1.CI.JNADH*O2conv/2,T1/60,J1.CII*O2conv/2,T1/60,J1.CIII.Jc3/2*O2conv/2,T1/60,J1.CIV*O2conv/2,'linewidth',4)
xlabel('Time (min)','fontsize',32)
ylabel('J_{ETC} (mol 2e^-/min/l_{mito})','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a8,'ylim',[-25 60])

yvar = 'JQ';
h9=figure;
a9=axes('parent',h9,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(T0/60,J0.CI.JQ,'linewidth',4)
plot(T1/60,J1.CI.JQ,'linewidth',4)
xlabel('Time (min)','fontsize',32)
ylabel('J_{Q} (nmol O_2/min/U CS)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')