%% Simulate Experiments from Vinnakota et al. 2015 and Plot Results
clear

% load model parameters
load params

% load data
load data

%% Initiale Model Parameters and ICs
xi0 = Model_ICs(p); % general model ICs
xi0(p.i.O2_x)=200e-6; % buffer [O2]

% define Temp, ionic strength and make corrections to dGs and pKs
p.T = 310.15; % 37C
p.I = 0.17; % M
p = dGpKcorr(p); % adjust free energies and pKs for ionic strength and temp

%% Set water/mito volume fraction (based on mito load)
p.Vol.H2O_cyt = 1/(0.06*p.Vol.rho_m*1e3)/1; % (l buffer / l mito) based on 0.06 mg/ml mitochondria

%% Initialize Integrator Options

% Mass matrix
M=eye(p.i.length);
M(p.i.ATP_x,:)=0;M(p.i.NADH_x,:)=0;M(p.i.UQH2_x,:)=0;M(p.i.c2_i,:)=0;M(p.i.PCr_c,:)=0;

% define integration absolute tolerances for matrix and cytosol SVs
AbsTol_vec=1e-6.*ones(1,p.i.length);

% set global integrator options
options = odeset('RelTol',1e-3,'AbsTol',AbsTol_vec,'Vectorized','on', ...
    'MassSingular','yes','Mass',M,'MStateDependence','none','BDF','on');
% Note: cannot use non-negativity constraint with system of DAEs
p.max_cputime = 60; % s

%% Set Vinnakota ICs
xi0(p.i.H_c) = 10^-7.2;
xi0(p.i.Mg_c) = 1e-3;
xi0(p.i.ATP_c) = 5e-3;
xi0(p.i.K_c) = 0.13;
xi0(p.i.K_x) = 0.06;
xi0(p.i.Pi_c) = 1e-3;

% set Apyrase titration vector
Apyrase = logspace(-3,0,41); % umol/min

%% Run Model Initialization for 1 mM Pi (60s incubation)
p.ATPase.Etot = 0;
[t_ic,x_ic] = ode15s(@DAEs,[0 60],xi0,options,p);
[f_ic, J_ic] = DAEs(t_ic',x_ic.',p);

% set ICs for 1 mM Pi simulations
xi1=x_ic(end,:);

%% Run 1 mM Pi Simulations for Duration of Experiment
for j=1:numel(Apyrase)
p.ATPase.Etot = Apyrase(j)*1e-6/1e-3/60;%*m1(j); % M/s;

[t{1}{j},x{1}{j}] = ode15s(@DAEs,[0 180],xi1,options,p);
[f{1}{j},J{1}{j}] = DAEs(t{1}{j}',x{1}{j}.',p);
end

%% Run Model Initialization for 5 mM Pi (60s incubation)
p.ATPase.Etot = 0;
xi0(p.i.Pi_c) = 5e-3;
[t_ic2,x_ic2] = ode15s(@DAEs,[0 60],xi0,options,p);
[f_ic2, J_ic2] = DAEs(t_ic2',x_ic2.',p);

% set ICs for 5 mM Pi simulations
xi2=x_ic2(end,:);

%% Run 5 mM Pi Simulations for Duration of Experiment
for j=1:numel(Apyrase)
p.ATPase.Etot = Apyrase(j)*1e-6/1e-3/60;%*m2(j); % M/s;
[t{2}{j},x{2}{j}] = ode15s(@DAEs,[0 180*1],xi2,options,p);
[f{2}{j},J{2}{j}] = DAEs(t{2}{j}',x{2}{j}.',p);
end

%% Store Model Output for Simulations

O2conv = p.Vol.rho_m*60*(0.06/0.337)/2*1e9; % converts mol O/s/l mito to nmol O2/min/U CS
for i=1:2
    for j=1:numel(Apyrase)
    JSO1(i,j) = J{i}{j}.CI.JSO(end); % mol/lmito/s
    JSO3(i,j) = J{i}{j}.CIII.JSO(end); % mol/lmito/s
    JH2O2(i,j) = J{i}{j}.CI.JH2O2(end); % mol/lmito/s
    Jscavenging(i,j) = J{i}{j}.H2O2scavenge(end); % M/s
    JSODx(i,j) = J{i}{j}.SODx(end); % M/s
    JSODc(i,j) = J{i}{j}.SODc(end); % M/s
    JH2O2perm(i,j) = J{i}{j}.H2O2perm(end); % M/s wrt to matrix H2O volume
    JO2(i,j) = J{i}{j}.CIV(end)*O2conv ; % nmol O2/min/U CS
    SOx(i,j) = x{i}{j}(end,p.i.SO_x); % M
    SOc(i,j) = x{i}{j}(end,p.i.SO_c); % M
    H2O2x(i,j) = x{i}{j}(end,p.i.H2O2_x); % M
    H2O2c(i,j) = x{i}{j}(end,p.i.H2O2_c); % M
    JDH(i,j) = J{i}{j}.MitoDH(end); % mol/lmito/s
    dPsi(i,j) = x{i}{j}(end,p.i.dPsi); % M
    NADH(i,j) = x{i}{j}(end,p.i.NADH_x)/p.Ntot; % M
    UQH2(i,j) = x{i}{j}(end,p.i.UQH2_x)/p.Qtot; % M
    c2(i,j) = x{i}{j}(end,p.i.c2_i)/p.Ctot; % M
    ATP(i,j) = x{i}{j}(end,p.i.ATP_c)*1e3; % mM
    ADP(i,j) = x{i}{j}(end,p.i.ADP_c)*1e3; % mM
    ATPx(i,j) = x{i}{j}(end,p.i.ATP_x)/p.Atot; % M
    ADPx(i,j) = x{i}{j}(end,p.i.ADP_x)/p.Atot; % M
    Pix(i,j) = x{i}{j}(end,p.i.Pi_x); % M
    pHx(i,j) = -log10(x{i}{j}(end,p.i.H_x)); % M
    pHc(i,j) = -log10(x{i}{j}(end,p.i.H_c)); % M
    end
end

%% Plot Results
for i=1:numel(data.label)
dlabels{i}=data.label{i};
end

yvar = 'dPsi';
h1=figure;
a=axes('parent',h1,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(JO2(1,:),dPsi(1,:),'b','linewidth',4)
plot(JO2(2,:),dPsi(2,:),'r','linewidth',4)
[k,i1,i2]=intersect(data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(:,1),data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(:,1));
a1=herrorbar(data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(i1,3)*1e3,'bo');
set(a1(1),'markersize',10,'linewidth',2,'markerfacecolor',[0 0 1],'markeredgecolor',[0 0 0])
set(a1(2),'markersize',10,'linewidth',2,'markerfacecolor',[0 0 1],'markeredgecolor',[0 0 0])
errorbar(data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(i2,3),'bo','markersize',15,'linewidth',2,'markerfacecolor',[0 0 1],'markeredgecolor',[0 0 0])
[k,i1,i2]=intersect(data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(:,1),data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(:,1));
a2=herrorbar(data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(i1,3)*1e3,'ro');
set(a2(1),'markersize',10,'linewidth',2,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0])
set(a2(2),'markersize',10,'linewidth',2,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0])
errorbar(data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(i2,3),'ro','markersize',15,'linewidth',2,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0])
xlabel('V_O_2 (nmol O_2 / min / U CS)','fontsize',32)
ylabel('\Delta\Psi (mV)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a,'xtick',[0 50 100 150],'xlim',[0 165])

yvar = 'NADH';
h2=figure;
a=axes('parent',h2,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(JO2(1,:),NADH(1,:),'b','linewidth',4)
plot(JO2(2,:),NADH(2,:),'r','linewidth',4)
[k,i1,i2]=intersect(data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(:,1),data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(:,1));
a1=herrorbar(data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(i1,3)*1e3,'bo');
set(a1(1),'markersize',10,'linewidth',2,'markerfacecolor',[0 0 1],'markeredgecolor',[0 0 0])
set(a1(2),'markersize',10,'linewidth',2,'markerfacecolor',[0 0 1],'markeredgecolor',[0 0 0])
errorbar(data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(i2,3),'bo','markersize',15,'linewidth',3,'markerfacecolor',[0 0 1],'markeredgecolor',[0 0 0])
[k,i1,i2]=intersect(data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(:,1),data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(:,1));
a2=herrorbar(data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(i1,3)*1e3,'ro');
set(a2(1),'markersize',10,'linewidth',2,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0])
set(a2(2),'markersize',10,'linewidth',2,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0])
errorbar(data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(i2,3),'ro','markersize',15,'linewidth',3,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0])
xlabel('V_O_2 (nmol O_2 / min / U CS)','fontsize',32)
ylabel('NADH (normalized)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a,'xtick',[0 50 100 150],'xlim',[0 165])

yvar = 'Cytc';
h3=figure;
a=axes('parent',h3,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(JO2(1,:),c2(1,:),'b','linewidth',4)
plot(JO2(2,:),c2(2,:),'r','linewidth',4)
[k,i1,i2]=intersect(data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(:,1),data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(:,1));
a1=herrorbar(data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(i1,3)*1e3,'bo');
set(a1(1),'markersize',10,'linewidth',2,'markerfacecolor',[0 0 1],'markeredgecolor',[0 0 0])
set(a1(2),'markersize',10,'linewidth',2,'markerfacecolor',[0 0 1],'markeredgecolor',[0 0 0])
errorbar(data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(i2,3),'bo','markersize',15,'linewidth',3,'markerfacecolor',[0 0 1],'markeredgecolor',[0 0 0])
[k,i1,i2]=intersect(data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(:,1),data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(:,1));
a2=herrorbar(data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(i1,3)*1e3,'ro');
set(a2(1),'markersize',10,'linewidth',2,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0])
set(a2(2),'markersize',10,'linewidth',2,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0])
errorbar(data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(i2,3),'ro','markersize',15,'linewidth',3,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0])
xlabel('V_O_2 (nmol O_2 / min / U CS)','fontsize',32)
ylabel('Cyt c^{2+} (normalized)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a,'xtick',[0 50 100 150],'xlim',[0 165])

yvar = 'ADP';
h4=figure;
a=axes('parent',h4,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(JO2(1,:),ADP(1,:),'b','linewidth',4)
plot(JO2(2,:),ADP(2,:),'r','linewidth',4)
[k,i1,i2]=intersect(data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(:,1),data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(:,1));
a1=herrorbar(data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(i1,3)*1e3,'bo');
set(a1(1),'markersize',10,'linewidth',2,'markerfacecolor',[0 0 1],'markeredgecolor',[0 0 0])
set(a1(2),'markersize',10,'linewidth',2,'markerfacecolor',[0 0 1],'markeredgecolor',[0 0 0])
errorbar(data.Y{find(strcmpi(dlabels,'JO2set1a.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,[yvar 'set1a.txt']))}(i2,3),'bo','markersize',15,'linewidth',3,'markerfacecolor',[0 0 1],'markeredgecolor',[0 0 0])
[k,i1,i2]=intersect(data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(:,1),data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(:,1));
a2=herrorbar(data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(i1,3)*1e3,'ro');
set(a2(1),'markersize',10,'linewidth',2,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0])
set(a2(2),'markersize',10,'linewidth',2,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0])
errorbar(data.Y{find(strcmpi(dlabels,'JO2set1b.txt'))}(i1,2)*1e3,data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(i2,2),data.Y{find(strcmpi(dlabels,[yvar 'set1b.txt']))}(i2,3),'ro','markersize',15,'linewidth',3,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0])
xlabel('V_O_2 (nmol O_2 / min / U CS)','fontsize',32)
ylabel('Buffer ADP (mM)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a,'xtick',[0 50 100 150],'xlim',[0 165])

yvar = 'UQH2';
h6=figure;
a6=axes('parent',h6,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(JO2(1,:),UQH2(1,:),'color',[0 0 1],'linewidth',4)
plot(JO2(2,:),UQH2(2,:),'color',[1 0 0],'linewidth',4)
xlabel('V_O_2 (nmol O_2 / min / U CS)','fontsize',32)
ylabel('UQH_2 (normalized)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a6,'xtick',[0 50 100 150],'xlim',[0 165])

yvar = 'pH';
h7=figure;
a7=axes('parent',h7,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(JO2(1,:),pHx(1,:),'color',[0 0 1],'linewidth',4)
plot(JO2(2,:),pHx(2,:),'color',[1 0 0],'linewidth',4)
xlabel('V_O_2 (nmol O_2 / min / U CS)','fontsize',32)
ylabel(' Mitochondrial pH','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a7,'xtick',[0 50 100 150],'xlim',[0 165],'ylim',[7.35 7.5])

yvar = 'JROS1';
h7=figure;
a7=axes('parent',h7,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(JO2(1,:),JSO1(1,:)/2*1e9*p.Vol.rho_m*60,'--','color',[0 0.25 1],'linewidth',4)
plot(JO2(2,:),JSO1(2,:)/2*1e9*p.Vol.rho_m*60,'--','color',[1 0.25 0],'linewidth',4)
plot(JO2(1,:),JSO3(1,:)/2*1e9*p.Vol.rho_m*60,'-','color',[0 0.25 1],'linewidth',4)
plot(JO2(2,:),JSO3(2,:)/2*1e9*p.Vol.rho_m*60,'-','color',[1 0.25 0],'linewidth',4)
plot(JO2(1,:),JH2O2(1,:)*1e9*p.Vol.rho_m*60,':','color',[0 0.25 1],'linewidth',4)
plot(JO2(2,:),JH2O2(2,:)*1e9*p.Vol.rho_m*60,':','color',[1 0.25 0],'linewidth',4)
xlabel('V_O_2 (nmol O_2 / min / U CS)','fontsize',32)
ylabel('J_{ROS} (nmol 2e^-/min/mg)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a7,'xtick',[0 50 100 150],'xlim',[0 150])
set(a7,'ylim',[0 0.6],'ytick',[0 0.2 0.4 0.6])

yvar = 'JScavenge';
h8=figure;
a8=axes('parent',h8,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(JO2(1,:),Jscavenging(1,:)*1e9*p.Vol.rho_m*p.Vol.H2O_mtx*60,'color',[0 0 1],'linewidth',4)
plot(JO2(2,:),Jscavenging(2,:)*1e9*p.Vol.rho_m*p.Vol.H2O_mtx*60,'color',[1 0 0],'linewidth',4)
xlabel('V_O_2 (nmol O_2 / min / U CS)','fontsize',32)
ylabel('J_{ROS} (nmol H_2O_2/min/mg)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a8,'xtick',[0 50 100 150],'xlim',[0 150])
set(a8,'ylim',[0.6 1.2],'ytick',[0.6 0.8 1.0 1.2])

yvar = 'JROS/JO2'; % normalized on a per e- basis
h9=figure;
a9=axes('parent',h9,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(JO2(1,:),Jscavenging(1,:)./(JO2(1,:)/O2conv)*100,'color',[0 0.25 1],'linewidth',4)
plot(JO2(2,:),Jscavenging(2,:)./(JO2(2,:)/O2conv)*100,'color',[1 0.25 0],'linewidth',4)
xlabel('V_O_2 (nmol O_2 / min / U CS)','fontsize',32)
ylabel('% electron leak','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
% set(a9,'xtick',[0 0.2 0.4 0.6 0.8 1.0])
set(a9,'xtick',[0 50 100 150],'xlim',[0 150])
set(a9,'ylim',[0 2],'ytick',[0 0.5 1.0 1.5 2.0])

yvar = 'dPsi_vs_ROS';
h10=figure;
a10=axes('parent',h10,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(dPsi(1,:),Jscavenging(1,:)*1e9*p.Vol.rho_m*p.Vol.H2O_mtx*60,'color',[0 0.25 1],'linewidth',4)
plot(dPsi(2,:),Jscavenging(2,:)*1e9*p.Vol.rho_m*p.Vol.H2O_mtx*60,'color',[1 0.25 0],'linewidth',4)
xlabel('\Delta\Psi','fontsize',32)
ylabel('J_{ROS}(nmol H_2O_2/min/mg)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a10,'xlim',[160 190],'xtick',[160 170 180 190])

yvar = 'SO';
h11=figure;
a11=axes('parent',h11,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(JO2(1,:),SOx(1,:)*1e12,'color',[0 0.25 1],'linewidth',4)
plot(JO2(2,:),SOx(2,:)*1e12,'color',[1 0.25 0],'linewidth',4)
xlabel('V_O_2 (nmol O_2 / min / U CS)','fontsize',32)
ylabel('[O_2^{.-}]_x (pM)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a11,'xtick',[0 50 100 150],'xlim',[0 150])

yvar = 'H2O2';
h12=figure;
a12=axes('parent',h12,'fontsize',28,'fontname','Times New Roman');
box on
hold on
plot(JO2(1,:),H2O2x(1,:)*1e9,'color',[0 0.25 1],'linewidth',3)
plot(JO2(2,:),H2O2x(2,:)*1e9,'color',[1 0.25 0],'linewidth',3)
xlabel('V_O_2 (nmol O_2 / min / U CS)','fontsize',32)
ylabel('[H_2O_2]_x (nM)','fontsize',32)
title([]),axis('tight'),set(gcf,'color','none'),set(gca,'color','none')
set(a12,'xtick',[0 50 100 150],'xlim',[0 150])