% dGibbs free energy of formation ionic strength and temperature correction
% pH correction is not considered since individual formation energies are
% not used and only H+ balanced reactions are used, thus cancelling out the
% pH terms; however, chemical potential of dGf and dHf as a function of
% protons are considered
% enthalpy of formation ionic strength correction
% enthalpies are assumed to be temperature independent (zero specific heat)
% note that these corrections are based on a dGf0 and dHf0 for I=0
% 
% pKa correction for ionic strength and temperature
%
% Copyright:
% Jason Bazil, Kalyan Vinnakota, and Dan Beard. April 2015
% Beard Laboratory
% Molecular and Integrative Physiology
% University of Michigan

function p = dGpKcorr(p)

% obtain Debye-Huckel Constant at temp=T
a = 1.10708 - 1.54508e-3.*p.T + 5.95584e-6.*p.T.^2;
dadT = 1.54508e-3 + 1.191168e-5.*p.T;

% apply temperature correction 1st (insufficient dHf data)
%p.dGf = p.dGf0.*(p.T./298.15) + (1-p.T./298.15).*p.dHf0;
p.dGf = p.dGf0;

% apply ionic strength corrections last
p.dGf = p.dGf - a.*p.R.*p.T.*(p.z.^2-p.NH).*p.I.^0.5./(1+1.6*p.I.^0.5);
% enthalpy correction
p.dHf = p.dHf0 + dadT.*p.R.*p.T.^2.*(p.z.^2-p.NH).*p.I.^0.5./(1+1.6*p.I.^0.5);

% ionic strength correction
zc = ones(size(p.z,2),1)*[1 1 1 2 2]; % cation series valence
zi=(p.z'*ones(1,5)+zc); % set species charge
z2 = p.z'.^2*ones(1,5) - zi.^2 + zc.^2;
p.KD = p.KD0.*10.^(a./2.3025*p.I.^0.5./(1+1.6*p.I.^0.5).*z2);

% temperature correction (currently, not used due to insufficient enthalpy
% information)
%p.KD = exp(log(p.KD) + p.dHf./p.R.*(1./298.15-1./p.T));